"""
Statistics dashboard for ZScheduler application
"""

import logging
from typing import List, Dict, Any
from datetime import datetime, timedelta

from PyQt6.QtWidgets import (
    QWidget, QVBoxLayout, QHBoxLayout, QLabel,
    QTabWidget, QTableWidget, QTableWidgetItem,
    QComboBox, QFrame, QSizePolicy, QGridLayout
)
from PyQt6.QtCore import Qt, QSize
from PyQt6.QtGui import QPainter, QColor, QPen, QBrush, QPainterPath

from zscheduler_app.scheduler.scheduler import Scheduler, Schedule

logger = logging.getLogger(__name__)

class StatsCard(QFrame):
    """A card displaying a single statistic with label"""

    def __init__(self, title: str, value: str, parent=None):
        super().__init__(parent)
        self.setFrameShape(QFrame.Shape.StyledPanel)
        self.setLineWidth(1)
        self.setSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Fixed)
        self.setMinimumHeight(120)

        layout = QVBoxLayout(self)

        # Title label
        title_label = QLabel(title)
        title_label.setAlignment(Qt.AlignmentFlag.AlignCenter)

        # Value label with large font
        value_label = QLabel(value)
        value_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
        font = value_label.font()
        font.setPointSize(24)
        font.setBold(True)
        value_label.setFont(font)

        layout.addWidget(title_label)
        layout.addWidget(value_label)

class PieChart(QWidget):
    """Simple pie chart widget"""

    def __init__(self, data: Dict[str, float], colors: Dict[str, str], parent=None):
        super().__init__(parent)
        self.data = data
        self.colors = colors
        self.setMinimumSize(200, 200)

    def paintEvent(self, event):
        painter = QPainter(self)
        painter.setRenderHint(QPainter.RenderHint.Antialiasing)

        # Calculate total value
        total = sum(self.data.values())
        if total <= 0:
            return

        # Calculate center and radius
        width = self.width()
        height = self.height()
        center_x = width / 2
        center_y = height / 2
        radius = min(width, height) / 2 - 10

        # Draw pie slices
        start_angle = 0
        for label, value in self.data.items():
            span_angle = 360 * (value / total)

            # Create and set brush color
            color = QColor(self.colors.get(label, "#CCCCCC"))
            painter.setBrush(QBrush(color))

            # Draw pie slice
            painter.drawPie(
                int(center_x - radius),
                int(center_y - radius),
                int(radius * 2),
                int(radius * 2),
                int(start_angle * 16),
                int(span_angle * 16)
            )

            start_angle += span_angle

class BarChart(QWidget):
    """Simple bar chart widget"""

    def __init__(self, data: Dict[str, float], color: str, parent=None):
        super().__init__(parent)
        self.data = data
        self.color = color
        self.setMinimumSize(300, 200)

    def paintEvent(self, event):
        painter = QPainter(self)
        painter.setRenderHint(QPainter.RenderHint.Antialiasing)

        # Get max value for scaling
        values = list(self.data.values())
        if not values:
            return

        max_value = max(values)
        if max_value <= 0:
            return

        # Calculate sizes
        width = self.width()
        height = self.height()
        bar_width = width / (len(self.data) * 2)
        bottom = height - 20
        scale_factor = (height - 40) / max_value

        # Draw bars
        painter.setPen(QPen(Qt.GlobalColor.black, 1))
        painter.setBrush(QBrush(QColor(self.color)))

        x = bar_width
        for label, value in self.data.items():
            bar_height = value * scale_factor
            painter.drawRect(
                int(x),
                int(bottom - bar_height),
                int(bar_width),
                int(bar_height)
            )

            # Draw label
            painter.drawText(
                int(x),
                int(bottom + 15),
                int(bar_width),
                20,
                Qt.AlignmentFlag.AlignCenter,
                label
            )

            x += bar_width * 2

class StatisticsDashboard(QWidget):
    """
    Widget displaying statistics and analytics about scheduled tasks.

    Shows execution success rates, frequency of runs, and other metrics
    to provide insights into the scheduler's performance.
    """

    def __init__(self, scheduler: Scheduler, parent=None):
        """
        Initialize the statistics dashboard.

        Args:
            scheduler: The scheduler instance
            parent: Parent widget
        """
        super().__init__(parent)
        self.scheduler = scheduler
        self.schedules = []  # Ensure schedules attribute always exists
        # Setup UI
        self._setup_ui()

    def _setup_ui(self):
        """Set up the UI components."""
        # Main layout
        main_layout = QVBoxLayout(self)

        # Period selection
        period_layout = QHBoxLayout()
        period_label = QLabel("Time Period:")
        self.period_combo = QComboBox()
        self.period_combo.addItems(["Last 24 Hours", "Last 7 Days", "Last 30 Days", "All Time"])
        self.period_combo.currentIndexChanged.connect(self._update_statistics)

        period_layout.addWidget(period_label)
        period_layout.addWidget(self.period_combo)
        period_layout.addStretch()

        main_layout.addLayout(period_layout)

        # Stats cards
        stats_layout = QHBoxLayout()

        self.total_tasks_card = StatsCard("Total Tasks", "0")
        self.active_tasks_card = StatsCard("Active Tasks", "0")
        self.executions_card = StatsCard("Total Executions", "0")
        self.success_rate_card = StatsCard("Success Rate", "0%")

        stats_layout.addWidget(self.total_tasks_card)
        stats_layout.addWidget(self.active_tasks_card)
        stats_layout.addWidget(self.executions_card)
        stats_layout.addWidget(self.success_rate_card)

        main_layout.addLayout(stats_layout)

        # Tabs for different charts
        self.chart_tabs = QTabWidget()

        # Task status tab
        status_tab = QWidget()
        status_layout = QVBoxLayout(status_tab)

        # Placeholder for charts
        self.status_chart = PieChart({
            "Active": 0,
            "Paused": 0,
            "Completed": 0
        }, {
            "Active": "#50fa7b",
            "Paused": "#ffb86c",
            "Completed": "#8be9fd"
        })

        status_layout.addWidget(self.status_chart)
        self.chart_tabs.addTab(status_tab, "Task Status")

        # Execution history tab
        history_tab = QWidget()
        history_layout = QVBoxLayout(history_tab)

        # Placeholder for execution history
        self.history_chart = BarChart({
            "Mon": 0,
            "Tue": 0,
            "Wed": 0,
            "Thu": 0,
            "Fri": 0,
            "Sat": 0,
            "Sun": 0
        }, "#50fa7b")

        history_layout.addWidget(self.history_chart)
        self.chart_tabs.addTab(history_tab, "Execution History")

        # Task types tab
        types_tab = QWidget()
        types_layout = QVBoxLayout(types_tab)

        # Placeholder for task types chart
        self.types_chart = PieChart({
            "Command": 0,
            "Python": 0
        }, {
            "Command": "#8be9fd",
            "Python": "#bd93f9"
        })

        types_layout.addWidget(self.types_chart)
        self.chart_tabs.addTab(types_tab, "Task Types")

        # Add tabs to main layout
        main_layout.addWidget(self.chart_tabs)

        # Recent executions table
        main_layout.addWidget(QLabel("Recent Executions"))

        self.executions_table = QTableWidget(0, 4)
        self.executions_table.setHorizontalHeaderLabels(["Task Name", "Status", "Execution Time", "Duration"])
        self.executions_table.horizontalHeader().setStretchLastSection(True)

        main_layout.addWidget(self.executions_table)

    def update_statistics(self, schedules):
        """
        Update the statistics dashboard with data from schedules.

        Args:
            schedules: List of schedules to analyze
        """
        self.schedules = schedules if schedules is not None else []
        self._update_statistics()

    def _update_statistics(self):
        """Update all statistics based on current data and selected period."""
        if not hasattr(self, 'schedules') or not self.schedules:
            # Defensive: clear UI if no schedules
            self._update_card_value(self.total_tasks_card, "0")
            self._update_card_value(self.active_tasks_card, "0")
            self._update_card_value(self.executions_card, "0")
            self._update_card_value(self.success_rate_card, "0%")
            self.status_chart.data = {"Active": 0, "Paused": 0, "Completed": 0}
            self.history_chart.data = {"Mon": 0, "Tue": 0, "Wed": 0, "Thu": 0, "Fri": 0, "Sat": 0, "Sun": 0}
            self.types_chart.data = {"Command": 0, "Python": 0}
            self.executions_table.setRowCount(0)
            self.status_chart.update()
            self.history_chart.update()
            self.types_chart.update()
            return

        # Get current time period
        period_index = self.period_combo.currentIndex()
        cutoff_date = datetime.now()

        if period_index == 0:  # Last 24 Hours
            cutoff_date -= timedelta(days=1)
        elif period_index == 1:  # Last 7 Days
            cutoff_date -= timedelta(days=7)
        elif period_index == 2:  # Last 30 Days
            cutoff_date -= timedelta(days=30)
        else:  # All Time
            cutoff_date = datetime.min

        # Calculate statistics
        total_tasks = len(self.schedules)
        active_tasks = sum(1 for s in self.schedules if not s.get('paused', False))

        # Count executions
        total_executions = 0
        success_count = 0

        for schedule in self.schedules:
            total_executions += schedule.get('execution_count', 0)
            success_count += schedule.get('success_count', 0)

        # Calculate success rate
        if total_executions > 0:
            success_rate = (success_count / total_executions) * 100
        else:
            success_rate = 0

        # Update stats cards
        self._update_card_value(self.total_tasks_card, str(total_tasks))
        self._update_card_value(self.active_tasks_card, str(active_tasks))
        self._update_card_value(self.executions_card, str(total_executions))
        self._update_card_value(self.success_rate_card, f"{success_rate:.1f}%")

        # Update status chart
        paused_tasks = sum(1 for s in self.schedules if s.get('paused', False))
        completed_tasks = 0  # No concept of "completed" in our simplified model

        self.status_chart.data = {
            "Active": active_tasks,
            "Paused": paused_tasks,
            "Completed": completed_tasks
        }

        # Create execution history data - using a simplified approach
        day_counts = {
            "Mon": 0, "Tue": 0, "Wed": 0, "Thu": 0,
            "Fri": 0, "Sat": 0, "Sun": 0
        }

        # In a real implementation, this would analyze actual execution timestamps
        # For now, just distribute executions evenly
        if total_executions > 0:
            base_count = total_executions // 7
            remainder = total_executions % 7

            for i, day in enumerate(day_counts.keys()):
                day_counts[day] = base_count + (1 if i < remainder else 0)

        self.history_chart.data = day_counts

        # Update task types chart - all are command tasks in our implementation
        self.types_chart.data = {
            "Command": total_tasks,
            "Python": 0
        }

        # Update executions table
        self.executions_table.setRowCount(0)

        # Add recent executions (simplified)
        row_count = min(10, total_tasks)  # Show up to 10 rows

        for i in range(row_count):
            if i >= len(self.schedules):
                break

            schedule = self.schedules[i]

            # Insert new row
            self.executions_table.insertRow(i)

            # Task name
            name_item = QTableWidgetItem(schedule.get('name', f"Task-{i+1}"))
            self.executions_table.setItem(i, 0, name_item)

            # Status - simplified
            status_text = "Paused" if schedule.get('paused', False) else "Active"
            status_item = QTableWidgetItem(status_text)
            status_color = "#ffb86c" if schedule.get('paused', False) else "#50fa7b"
            status_item.setForeground(QColor(status_color))
            self.executions_table.setItem(i, 1, status_item)

            # Execution time - use last_run if available
            last_run = schedule.get('last_run', None)
            if last_run:
                try:
                    dt = datetime.fromisoformat(last_run)
                    time_str = dt.strftime("%Y-%m-%d %H:%M:%S")
                except (ValueError, TypeError):
                    time_str = "Never"
            else:
                time_str = "Never"

            time_item = QTableWidgetItem(time_str)
            self.executions_table.setItem(i, 2, time_item)

            # Duration - not tracked in our model
            duration_item = QTableWidgetItem("N/A")
            self.executions_table.setItem(i, 3, duration_item)

        # Repaint charts
        self.status_chart.update()
        self.history_chart.update()
        self.types_chart.update()

        # Update status bar (defensive)
        main_window = self.window()
        if hasattr(main_window, "statusBar"):
            try:
                main_window.statusBar().showMessage(str(len(self.schedules)) + " schedules loaded")
            except Exception:
                pass  # Don't let status bar errors break the dashboard

    def _update_card_value(self, card, value):
        """Update the value displayed on a stats card"""
        # Find the QLabel child that shows the value (second label)
        labels = card.findChildren(QLabel)
        if len(labels) >= 2:
            labels[1].setText(value)



"""
Time parsing utilities for ZScheduler
"""

import re
from datetime import datetime, timedelta
from typing import Optional, Union, Dict, Any

import dateutil.parser
from dateutil.relativedelta import relativedelta

class TimeParser:
    """
    Utility class for parsing various time formats into datetime objects.

    Supports:
    - Absolute dates and times ("May 25, 2025 at 10:00 PM")
    - Relative times ("3 hours from now", "19 hours from now")
    - Simple time formats ("10:30", "10:30 PM")
    """

    @staticmethod
    def parse(time_str: str) -> datetime:
        """
        Parse a time string into a datetime object.

        Args:
            time_str: Time string to parse

        Returns:
            Parsed datetime object

        Raises:
            ValueError: If the time string cannot be parsed
        """
        # Try natural language relative time first
        relative_time = TimeParser.parse_relative_time(time_str)
        if relative_time is not None:
            return relative_time

        # Try simple time format (just the time)
        simple_time = TimeParser.parse_simple_time(time_str)
        if simple_time is not None:
            return simple_time

        # Fall back to dateutil parser
        try:
            return dateutil.parser.parse(time_str)
        except (ValueError, OverflowError):
            raise ValueError(f"Could not parse time string: {time_str}")

    @staticmethod
    def parse_relative_time(time_str: str) -> Optional[datetime]:
        """
        Parse a relative time string like "3 hours from now".

        Args:
            time_str: Time string to parse

        Returns:
            Parsed datetime object or None if not a relative time
        """
        # Pattern for "X units from now"
        pattern = r"(\d+)\s+(second|minute|hour|day|week|month|year)s?\s+from\s+now"
        match = re.match(pattern, time_str, re.IGNORECASE)
        if not match:
            return None

        value = int(match.group(1))
        unit = match.group(2).lower()

        now = datetime.now()

        # Handle different units
        if unit == "second":
            return now + timedelta(seconds=value)
        elif unit == "minute":
            return now + timedelta(minutes=value)
        elif unit == "hour":
            return now + timedelta(hours=value)
        elif unit == "day":
            return now + timedelta(days=value)
        elif unit == "week":
            return now + timedelta(weeks=value)
        elif unit == "month":
            return now + relativedelta(months=value)
        elif unit == "year":
            return now + relativedelta(years=value)

        return None

    @staticmethod
    def parse_simple_time(time_str: str) -> Optional[datetime]:
        """
        Parse a simple time format like "10:30" or "10:30 PM".

        Args:
            time_str: Time string to parse

        Returns:
            Parsed datetime object or None if not a simple time
        """
        # Pattern for "HH:MM" or "HH:MM AM/PM"
        pattern = r"(\d{1,2}):(\d{2})(?:\s*(AM|PM))?"
        match = re.match(pattern, time_str, re.IGNORECASE)
        if not match:
            return None

        hour = int(match.group(1))
        minute = int(match.group(2))
        am_pm = match.group(3)

        # Adjust hour for 12-hour clock
        if am_pm:
            if am_pm.upper() == "PM" and hour < 12:
                hour += 12
            elif am_pm.upper() == "AM" and hour == 12:
                hour = 0

        # Use today's date with the specified time
        now = datetime.now().replace(hour=hour, minute=minute, second=0, microsecond=0)

        # If the time is already passed today, schedule for tomorrow
        if now < datetime.now():
            now += timedelta(days=1)

        return now



"""
JSON storage implementation for persisting schedules
"""

import os
import json
import logging
from typing import Dict, List, Any, Optional

from ..core.schedule import Schedule
from ..tasks.task import Task
from ..tasks.command_task import CommandTask
from ..tasks.python_task import PythonTask

logger = logging.getLogger(__name__)

class JsonStore:
    """
    JSON-based storage for persisting schedules to disk.

    This class handles saving and loading schedules from a JSON file.
    """

    def __init__(self, file_path: str):
        """
        Initialize a new JSON store.

        Args:
            file_path: Path to the JSON file for storage
        """
        self.file_path = file_path
        self.makedirs()

    def makedirs(self) -> None:
        """Ensure the directory for the file exists."""
        directory = os.path.dirname(self.file_path)
        if directory and not os.path.exists(directory):
            os.makedirs(directory)

    def save_schedules(self, schedules: List[Schedule]) -> None:
        """
        Save schedules to the JSON file.

        Args:
            schedules: List of schedules to save
        """
        try:
            data = {
                "version": 1,
                "schedules": [schedule.to_dict() for schedule in schedules]
            }

            with open(self.file_path, 'w') as f:
                json.dump(data, f, indent=2)

            logger.info(f"Saved {len(schedules)} schedules to {self.file_path}")

        except Exception as e:
            logger.exception(f"Error saving schedules to {self.file_path}: {e}")

    def load_schedules(self) -> List[Schedule]:
        """
        Load schedules from the JSON file.

        Returns:
            List of loaded schedules
        """
        if not os.path.exists(self.file_path):
            logger.info(f"No schedules file found at {self.file_path}")
            return []

        try:
            with open(self.file_path, 'r') as f:
                data = json.load(f)

            version = data.get("version", 1)
            schedules_data = data.get("schedules", [])

            schedules = []
            for schedule_data in schedules_data:
                try:
                    schedule = self._load_schedule(schedule_data)
                    schedules.append(schedule)
                except Exception as e:
                    logger.error(f"Error loading schedule {schedule_data.get('id')}: {e}")

            logger.info(f"Loaded {len(schedules)} schedules from {self.file_path}")
            return schedules

        except Exception as e:
            logger.exception(f"Error loading schedules from {self.file_path}: {e}")
            return []

    def _load_schedule(self, data: Dict[str, Any]) -> Schedule:
        """
        Load a single schedule from its dictionary representation.

        Args:
            data: Dictionary containing schedule data

        Returns:
            Loaded Schedule object
        """
        task_data = data["task"]
        task_type = task_data["type"]

        # Create the appropriate task type
        if task_type == "CommandTask":
            task = CommandTask.from_dict(task_data)
        elif task_type == "PythonTask":
            task = PythonTask.from_dict(task_data)
        else:
            raise ValueError(f"Unknown task type: {task_type}")

        # Create the schedule
        schedule = Schedule(
            task=task,
            trigger_type=data["trigger_type"],
            trigger_args=data["trigger_args"],
            name=data.get("name"),
            max_runs=data.get("max_runs")
        )

        # Set additional attributes
        schedule.id = data["id"]

        if data.get("created_at"):
            from datetime import datetime
            schedule.created_at = datetime.fromisoformat(data["created_at"])

        if data.get("last_run"):
            schedule.last_run = datetime.fromisoformat(data["last_run"])

        schedule.run_count = data.get("run_count", 0)
        schedule.paused = data.get("paused", False)

        return schedule



"""
JSON file-based storage for ZScheduler data
"""

import os
import json
import logging
from pathlib import Path
from typing import List, Dict, Any, Optional

logger = logging.getLogger(__name__)

class JsonStore:
    """
    JSON file-based storage for schedules
    """

    def __init__(self, file_path: str):
        """
        Initialize the JSON store

        Args:
            file_path: Path to the JSON file
        """
        self.file_path = Path(file_path)
        self._ensure_parent_dir()

    def _ensure_parent_dir(self) -> None:
        """Ensure the parent directory exists"""
        parent_dir = self.file_path.parent
        if not parent_dir.exists():
            parent_dir.mkdir(parents=True, exist_ok=True)
            logger.info("Created directory: " + str(parent_dir))

    def load(self) -> List[Dict[str, Any]]:
        """
        Load schedules from the JSON file

        Returns:
            List of schedule dictionaries
        """
        if not self.file_path.exists():
            logger.info(f"Store file does not exist: {self.file_path}")
            return []

        try:
            with open(self.file_path, 'r', encoding='utf-8') as f:
                data = json.load(f)

            if not isinstance(data, list):
                logger.warning(f"Invalid data format in {self.file_path}. Expected list, got {type(data).__name__}")
                return []

            logger.info(f"Loaded {len(data)} schedules from {self.file_path}")
            return data
        except json.JSONDecodeError as e:
            logger.error(f"Failed to parse JSON from {self.file_path}: {e}")
            return []
        except Exception as e:
            logger.error(f"Error loading from {self.file_path}: {e}")
            return []

    def save(self, schedules: List[Dict[str, Any]]) -> bool:
        """
        Save schedules to the JSON file

        Args:
            schedules: List of schedule dictionaries

        Returns:
            True if successful, False otherwise
        """
        self._ensure_parent_dir()

        try:
            # Make sure file is completely overwritten
            with open(self.file_path, 'w', encoding='utf-8') as f:
                # Clear the file contents first
                f.truncate(0)
                # Write the new schedules
                json.dump(schedules, f, indent=2)

            logger.info(f"Saved {len(schedules)} schedules to {self.file_path}")
            return True
        except Exception as e:
            logger.error(f"Error saving to {self.file_path}: {e}")
            return False

    def get_file_path(self) -> str:
        """
        Get the file path

        Returns:
            String representation of the file path
        """
        return str(self.file_path)

    def delete_file(self) -> bool:
        """
        Delete the JSON file

        Returns:
            True if successful, False otherwise
        """
        try:
            if self.file_path.exists():
                self.file_path.unlink()
                logger.info(f"Deleted file: {self.file_path}")
                return True
            else:
                logger.info(f"File does not exist, nothing to delete: {self.file_path}")
                return True
        except Exception as e:
            logger.error(f"Error deleting file {self.file_path}: {e}")
            return False



"""
Theme Manager for ZScheduler UI
"""

from typing import Dict, Any, Optional

from PyQt6.QtWidgets import QApplication
from PyQt6.QtGui import QPalette, QColor
from PyQt6.QtCore import Qt

class ThemeManager:
    """
    Manages application theming and color schemes.

    This class provides functionality to apply dark and light themes
    with the specified color schemes to the entire application.
    """

    # Default theme colors
    DEFAULT_DARK_THEME = {
        "background": "#0a192f",
        "text": "#f1fa8c",
        "accent": "#50fa7b",
        "info": "#8be9fd",
        "warning": "#ffb86c",
        "error": "#ff5555"
    }

    DEFAULT_LIGHT_THEME = {
        "background": "#e0f2e9",
        "text": "#0a192f",
        "accent": "#1a365d",
        "info": "#3182ce",
        "warning": "#dd6b20",
        "error": "#e53e3e"
    }

    def __init__(self, config):
        """
        Initialize the theme manager.

        Args:
            config: Application configuration object
        """
        self.config = config
        self.current_theme = config.get("ui.theme", "dark")

    def get_theme_colors(self) -> Dict[str, str]:
        """
        Get the current theme colors.

        Returns:
            Dictionary of color values for the theme
        """
        if self.current_theme == "dark":
            colors = self.DEFAULT_DARK_THEME.copy()
            # Override with any config-specific colors if available
            config_colors = self.config.get("ui.theme.dark_colors", {})
            colors.update(config_colors)
        else:
            colors = self.DEFAULT_LIGHT_THEME.copy()
            # Override with any config-specific colors if available
            config_colors = self.config.get("ui.theme.light_colors", {})
            colors.update(config_colors)

        return colors

    def apply_theme(self, theme_name: str) -> None:
        """
        Apply the specified theme to the application.

        Args:
            theme_name: Theme name ('dark', 'light', or 'system')
        """
        # Handle system theme
        if theme_name == "system":
            # Default to dark theme for now - could detect system theme in a real implementation
            theme_name = "dark"

        self.current_theme = theme_name

        # Get theme colors
        colors = self.get_theme_colors()

        # Apply the theme
        if theme_name == "dark":
            self._apply_dark_theme(colors)
        else:
            self._apply_light_theme(colors)

    def _apply_dark_theme(self, colors: Dict[str, str]) -> None:
        """
        Apply the dark theme to the application.

        Args:
            colors: Dictionary of color values for the theme
        """
        app = QApplication.instance()
        if not app:
            return

        # Create a dark palette
        palette = QPalette()

        # Set colors based on the theme
        bg_color = QColor(colors.get("background", "#0a192f"))
        text_color = QColor(colors.get("text", "#f1fa8c"))
        accent_color = QColor(colors.get("accent", "#50fa7b"))

        # Base colors
        palette.setColor(QPalette.ColorRole.Window, bg_color)
        palette.setColor(QPalette.ColorRole.WindowText, text_color)
        palette.setColor(QPalette.ColorRole.Base, QColor(bg_color).darker(110))
        palette.setColor(QPalette.ColorRole.AlternateBase, bg_color)
        palette.setColor(QPalette.ColorRole.ToolTipBase, bg_color)
        palette.setColor(QPalette.ColorRole.ToolTipText, text_color)
        palette.setColor(QPalette.ColorRole.Text, text_color)
        palette.setColor(QPalette.ColorRole.Button, bg_color)
        palette.setColor(QPalette.ColorRole.ButtonText, text_color)
        palette.setColor(QPalette.ColorRole.BrightText, Qt.GlobalColor.white)

        # Disabled colors
        palette.setColor(QPalette.ColorGroup.Disabled, QPalette.ColorRole.WindowText, QColor(text_color).darker(150))
        palette.setColor(QPalette.ColorGroup.Disabled, QPalette.ColorRole.Text, QColor(text_color).darker(150))
        palette.setColor(QPalette.ColorGroup.Disabled, QPalette.ColorRole.ButtonText, QColor(text_color).darker(150))

        # Highlight colors
        palette.setColor(QPalette.ColorRole.Highlight, accent_color)
        palette.setColor(QPalette.ColorRole.HighlightedText, QColor(bg_color))

        # Link colors
        palette.setColor(QPalette.ColorRole.Link, QColor(colors.get("info", "#8be9fd")))
        palette.setColor(QPalette.ColorRole.LinkVisited, QColor(colors.get("info", "#8be9fd")).darker(120))

        # Apply palette to application
        app.setPalette(palette)

        # Apply stylesheet for additional styling
        app.setStyleSheet(self._get_dark_stylesheet(colors))

    def _apply_light_theme(self, colors: Dict[str, str]) -> None:
        """
        Apply the light theme to the application.

        Args:
            colors: Dictionary of color values for the theme
        """
        app = QApplication.instance()
        if not app:
            return

        # Create a light palette
        palette = QPalette()

        # Set colors based on the theme
        bg_color = QColor(colors.get("background", "#e0f2e9"))
        text_color = QColor(colors.get("text", "#0a192f"))
        accent_color = QColor(colors.get("accent", "#1a365d"))

        # Base colors
        palette.setColor(QPalette.ColorRole.Window, bg_color)
        palette.setColor(QPalette.ColorRole.WindowText, text_color)
        palette.setColor(QPalette.ColorRole.Base, QColor(bg_color).lighter(110))
        palette.setColor(QPalette.ColorRole.AlternateBase, bg_color)
        palette.setColor(QPalette.ColorRole.ToolTipBase, bg_color)
        palette.setColor(QPalette.ColorRole.ToolTipText, text_color)
        palette.setColor(QPalette.ColorRole.Text, text_color)
        palette.setColor(QPalette.ColorRole.Button, bg_color)
        palette.setColor(QPalette.ColorRole.ButtonText, text_color)
        palette.setColor(QPalette.ColorRole.BrightText, Qt.GlobalColor.black)

        # Disabled colors
        palette.setColor(QPalette.ColorGroup.Disabled, QPalette.ColorRole.WindowText, QColor(text_color).lighter(150))
        palette.setColor(QPalette.ColorGroup.Disabled, QPalette.ColorRole.Text, QColor(text_color).lighter(150))
        palette.setColor(QPalette.ColorGroup.Disabled, QPalette.ColorRole.ButtonText, QColor(text_color).lighter(150))

        # Highlight colors
        palette.setColor(QPalette.ColorRole.Highlight, accent_color)
        palette.setColor(QPalette.ColorRole.HighlightedText, QColor(bg_color))

        # Link colors
        palette.setColor(QPalette.ColorRole.Link, QColor(colors.get("info", "#3182ce")))
        palette.setColor(QPalette.ColorRole.LinkVisited, QColor(colors.get("info", "#3182ce")).darker(120))

        # Apply palette to application
        app.setPalette(palette)

        # Apply stylesheet for additional styling
        app.setStyleSheet(self._get_light_stylesheet(colors))

    def _get_dark_stylesheet(self, colors: Dict[str, str]) -> str:
        """
        Get the dark theme stylesheet.

        Args:
            colors: Dictionary of color values for the theme

        Returns:
            CSS stylesheet string
        """
        return f"""
        QMainWindow, QDialog, QWidget {{
            background-color: {colors.get("background", "#0a192f")};
            color: {colors.get("text", "#f1fa8c")};
        }}

        QMenuBar, QMenu {{
            background-color: {QColor(colors.get("background", "#0a192f")).darker(110).name()};
            color: {colors.get("text", "#f1fa8c")};
        }}

        QMenuBar::item:selected, QMenu::item:selected {{
            background-color: {colors.get("accent", "#50fa7b")};
            color: {colors.get("background", "#0a192f")};
        }}

        QTabWidget::pane {{
            border: 1px solid {QColor(colors.get("accent", "#50fa7b")).darker(150).name()};
        }}

        QTabBar::tab {{
            background-color: {QColor(colors.get("background", "#0a192f")).darker(120).name()};
            color: {colors.get("text", "#f1fa8c")};
            padding: 8px 12px;
            border-top-left-radius: 4px;
            border-top-right-radius: 4px;
        }}

        QTabBar::tab:selected {{
            background-color: {colors.get("accent", "#50fa7b")};
            color: {colors.get("background", "#0a192f")};
        }}

        QPushButton {{
            background-color: {QColor(colors.get("background", "#0a192f")).darker(120).name()};
            color: {colors.get("text", "#f1fa8c")};
            border: 1px solid {colors.get("accent", "#50fa7b")};
            padding: 6px 12px;
            border-radius: 4px;
        }}

        QPushButton:hover {{
            background-color: {QColor(colors.get("accent", "#50fa7b")).darker(120).name()};
            color: {colors.get("background", "#0a192f")};
        }}

        QPushButton:pressed {{
            background-color: {colors.get("accent", "#50fa7b")};
            color: {colors.get("background", "#0a192f")};
        }}

        QLineEdit, QTextEdit, QPlainTextEdit, QComboBox, QSpinBox, QDateTimeEdit {{
            background-color: {QColor(colors.get("background", "#0a192f")).darker(110).name()};
            color: {colors.get("text", "#f1fa8c")};
            border: 1px solid {QColor(colors.get("accent", "#50fa7b")).darker(150).name()};
            border-radius: 4px;
            padding: 4px;
        }}

        QTableView, QTreeView, QListView {{
            background-color: {QColor(colors.get("background", "#0a192f")).darker(110).name()};
            color: {colors.get("text", "#f1fa8c")};
            alternate-background-color: {QColor(colors.get("background", "#0a192f")).darker(120).name()};
            border: 1px solid {QColor(colors.get("accent", "#50fa7b")).darker(150).name()};
        }}

        QHeaderView::section {{
            background-color: {QColor(colors.get("background", "#0a192f")).darker(130).name()};
            color: {colors.get("text", "#f1fa8c")};
            padding: 4px;
            border: 1px solid {QColor(colors.get("accent", "#50fa7b")).darker(150).name()};
        }}

        QScrollBar {{
            background-color: {QColor(colors.get("background", "#0a192f")).darker(110).name()};
            border-radius: 4px;
        }}

        QScrollBar::handle {{
            background-color: {QColor(colors.get("accent", "#50fa7b")).darker(150).name()};
            border-radius: 4px;
        }}

        QToolTip {{
            background-color: {colors.get("background", "#0a192f")};
            color: {colors.get("text", "#f1fa8c")};
            border: 1px solid {colors.get("accent", "#50fa7b")};
            padding: 4px;
        }}
        """

    def _get_light_stylesheet(self, colors: Dict[str, str]) -> str:
        """
        Get the light theme stylesheet.

        Args:
            colors: Dictionary of color values for the theme

        Returns:
            CSS stylesheet string
        """
        return f"""
        QMainWindow, QDialog, QWidget {{
            background-color: {colors.get("background", "#e0f2e9")};
            color: {colors.get("text", "#0a192f")};
        }}

        QMenuBar, QMenu {{
            background-color: {QColor(colors.get("background", "#e0f2e9")).darker(110).name()};
            color: {colors.get("text", "#0a192f")};
        }}

        QMenuBar::item:selected, QMenu::item:selected {{
            background-color: {colors.get("accent", "#1a365d")};
            color: {colors.get("background", "#e0f2e9")};
        }}

        QTabWidget::pane {{
            border: 1px solid {QColor(colors.get("accent", "#1a365d")).lighter(150).name()};
        }}

        QTabBar::tab {{
            background-color: {QColor(colors.get("background", "#e0f2e9")).darker(110).name()};
            color: {colors.get("text", "#0a192f")};
            padding: 8px 12px;
            border-top-left-radius: 4px;
            border-top-right-radius: 4px;
        }}

        QTabBar::tab:selected {{
            background-color: {colors.get("accent", "#1a365d")};
            color: {colors.get("background", "#e0f2e9")};
        }}

        QPushButton {{
            background-color: {QColor(colors.get("background", "#e0f2e9")).darker(110).name()};
            color: {colors.get("text", "#0a192f")};
            border: 1px solid {colors.get("accent", "#1a365d")};
            padding: 6px 12px;
            border-radius: 4px;
        }}

        QPushButton:hover {{
            background-color: {QColor(colors.get("accent", "#1a365d")).lighter(120).name()};
            color: {colors.get("background", "#e0f2e9")};
        }}

        QPushButton:pressed {{
            background-color: {colors.get("accent", "#1a365d")};
            color: {colors.get("background", "#e0f2e9")};
        }}

        QLineEdit, QTextEdit, QPlainTextEdit, QComboBox, QSpinBox, QDateTimeEdit {{
            background-color: {QColor(colors.get("background", "#e0f2e9")).lighter(110).name()};
            color: {colors.get("text", "#0a192f")};
            border: 1px solid {QColor(colors.get("accent", "#1a365d")).lighter(150).name()};
            border-radius: 4px;
            padding: 4px;
        }}

        QTableView, QTreeView, QListView {{
            background-color: {QColor(colors.get("background", "#e0f2e9")).lighter(105).name()};
            color: {colors.get("text", "#0a192f")};
            alternate-background-color: {QColor(colors.get("background", "#e0f2e9")).darker(105).name()};
            border: 1px solid {QColor(colors.get("accent", "#1a365d")).lighter(150).name()};
        }}

        QHeaderView::section {{
            background-color: {QColor(colors.get("background", "#e0f2e9")).darker(110).name()};
            color: {colors.get("text", "#0a192f")};
            padding: 4px;
            border: 1px solid {QColor(colors.get("accent", "#1a365d")).lighter(150).name()};
        }}

        QScrollBar {{
            background-color: {QColor(colors.get("background", "#e0f2e9")).lighter(105).name()};
            border-radius: 4px;
        }}

        QScrollBar::handle {{
            background-color: {QColor(colors.get("accent", "#1a365d")).lighter(150).name()};
            border-radius: 4px;
        }}

        QToolTip {{
            background-color: {colors.get("background", "#e0f2e9")};
            color: {colors.get("text", "#0a192f")};
            border: 1px solid {colors.get("accent", "#1a365d")};
            padding: 4px;
        }}
        """

    def toggle_theme(self) -> None:
        """
        Toggle between dark and light themes.
        """
        next_theme = "light" if self.current_theme == "dark" else "dark"
        self.apply_theme(next_theme)



"""
Application configuration management for ZScheduler
"""

import os
import json
import logging
from typing import Any, Dict, Optional

logger = logging.getLogger(__name__)

class AppConfig:
    """
    Manages application configuration and settings.

    Handles loading and saving settings from/to a JSON file.
    Provides defaults for settings that aren't specified.
    """

    # Default configuration values
    DEFAULT_CONFIG = {
        "ui": {
            "theme": "dark",  # 'dark' or 'light'
            "default_view": "list",  # 'list', 'calendar', 'timeline', etc.
            "refresh_interval": 1000,  # milliseconds
            "confirm_delete": True,
            "show_toolbar": True,
            "show_statusbar": True,
            "minimize_to_tray": True,
            "minimize_on_close": False
        },
        "window": {
            "width": 1200,
            "height": 800,
            "x": None,
            "y": None,
            "maximized": False
        },
        "ui.theme.dark_colors": {
            "background": "#0a192f",  # Deep blue
            "text": "#f1fa8c",        # Neon yellow
            "accent": "#50fa7b",      # Light green
            "success": "#50fa7b",     # Light green
            "warning": "#ffb86c",     # Orange
            "error": "#ff5555",       # Red
            "info": "#8be9fd"         # Cyan
        },
        "ui.theme.light_colors": {
            "background": "#e0f2e9",  # Light green
            "text": "#0a192f",        # Dark blue
            "accent": "#1a365d",      # Deeper blue
            "success": "#38a169",     # Green
            "warning": "#dd6b20",     # Orange
            "error": "#e53e3e",       # Red
            "info": "#3182ce"         # Blue
        },
        "scheduler": {
            "auto_start": True
        }
    }

    def __init__(self, config_path: str):
        """
        Initialize configuration manager.

        Args:
            config_path: Path to the configuration file
        """
        self.config_path = config_path
        self.config = self.DEFAULT_CONFIG.copy()
        self.load()

    def load(self) -> None:
        """Load configuration from file, or use defaults if file doesn't exist."""
        if not os.path.exists(self.config_path):
            logger.info("No configuration file found at " + str(self.config_path) + ", using defaults")
            return

        try:
            with open(self.config_path, 'r') as f:
                stored_config = json.load(f)

            # Update default config with stored values (keeping defaults for missing values)
            self._update_recursive(self.config, stored_config)
            logger.info("Loaded configuration from " + str(self.config_path))

        except Exception as e:
            logger.error("Error loading configuration from " + str(self.config_path) + ": " + str(e))

    def save(self) -> None:
        """Save current configuration to file."""
        try:
            # Ensure directory exists
            os.makedirs(os.path.dirname(self.config_path), exist_ok=True)

            with open(self.config_path, 'w') as f:
                json.dump(self.config, f, indent=2)

            logger.info("Saved configuration to " + str(self.config_path))

        except Exception as e:
            logger.error("Error saving configuration to " + str(self.config_path) + ": " + str(e))

    def get(self, key: str, default: Any = None) -> Any:
        """
        Get a configuration value by key.

        Args:
            key: Dot-separated key path (e.g., 'window.width')
            default: Default value if key is not found

        Returns:
            Configuration value or default
        """
        try:
            parts = key.split('.')
            value = self.config
            for part in parts:
                value = value[part]
            return value
        except (KeyError, TypeError):
            return default

    def set(self, key: str, value: Any) -> None:
        """
        Set a configuration value.

        Args:
            key: Dot-separated key path (e.g., 'window.width')
            value: Value to set
        """
        parts = key.split('.')
        target = self.config

        # Navigate to the containing dictionary
        for part in parts[:-1]:
            if part not in target or not isinstance(target[part], dict):
                target[part] = {}
            target = target[part]

        # Set the value
        target[parts[-1]] = value

    def _update_recursive(self, target: Dict[str, Any], source: Dict[str, Any]) -> None:
        """
        Recursively update a nested dictionary.

        Args:
            target: Dictionary to update
            source: Dictionary with new values
        """
        for key, value in source.items():
            if key in target and isinstance(target[key], dict) and isinstance(value, dict):
                self._update_recursive(target[key], value)
            else:
                target[key] = value

    def get_theme_colors(self) -> Dict[str, str]:
        """
        Get the current theme's color palette.

        Returns:
            Dictionary of color values for the current theme
        """
        theme = self.get('theme', 'dark')
        colors_key = 'colors.' + theme
        return self.get(colors_key, {})



"""
Schedule class for ZScheduler - represents a scheduled task
"""

import uuid
import logging
from datetime import datetime
from typing import Any, Dict, Optional

from ..tasks.task import Task

logger = logging.getLogger(__name__)

class Schedule:
    """
    Represents a scheduled task with execution metadata.

    A Schedule combines a Task with scheduling information and
    keeps track of execution history and status.
    """

    def __init__(
        self,
        task: Task,
        trigger_type: str,
        trigger_args: Dict[str, Any],
        name: Optional[str] = None,
        max_runs: Optional[int] = None
    ):
        """
        Initialize a new schedule.

        Args:
            task: The task to execute
            trigger_type: The trigger type (date, interval, cron)
            trigger_args: Arguments for the trigger
            name: Optional name for the schedule
            max_runs: Maximum number of times to run the task
        """
        self.id = str(uuid.uuid4())
        self.task = task
        self.name = name
        self.trigger_type = trigger_type
        self.trigger_args = trigger_args
        self.max_runs = max_runs

        self.created_at = datetime.now()
        self.last_run: Optional[datetime] = None
        self.next_run: Optional[datetime] = None
        self.run_count = 0
        self.paused = False
        self.execution_history = []

    def execute(self) -> None:
        """
        Execute the scheduled task.

        This method is called by the scheduler when the trigger fires.
        It handles execution, error handling, and housekeeping.
        """
        if self.max_runs is not None and self.run_count >= self.max_runs:
            logger.info(f"Schedule {self.id} reached maximum runs ({self.max_runs}), skipping execution")
            return

        self.last_run = datetime.now()
        self.run_count += 1

        start_time = datetime.now()
        execution_id = str(uuid.uuid4())

        logger.info(f"Executing schedule {self.id} (run {self.run_count})")

        execution_record = {
            "id": execution_id,
            "start_time": start_time,
            "success": False,
            "error": None,
            "result": None,
            "duration": None
        }

        try:
            result = self.task.execute()
            execution_record["result"] = result
            execution_record["success"] = True

            logger.info(f"Schedule {self.id} executed successfully")

        except Exception as e:
            logger.exception(f"Error executing schedule {self.id}: {e}")
            execution_record["error"] = str(e)

        finally:
            end_time = datetime.now()
            duration = (end_time - start_time).total_seconds()
            execution_record["end_time"] = end_time
            execution_record["duration"] = duration

            self.execution_history.append(execution_record)

            # Limit history size
            if len(self.execution_history) > 100:
                self.execution_history = self.execution_history[-100:]

        return

    def get_status(self) -> Dict[str, Any]:
        """
        Get the current status of this schedule.

        Returns:
            A dictionary containing the schedule status
        """
        return {
            "id": self.id,
            "name": self.name,
            "created_at": self.created_at,
            "last_run": self.last_run,
            "next_run": self.next_run,
            "run_count": self.run_count,
            "paused": self.paused,
            "max_runs": self.max_runs,
            "trigger_type": self.trigger_type,
            "task_type": self.task.__class__.__name__,
        }

    def to_dict(self) -> Dict[str, Any]:
        """
        Convert the schedule to a dictionary for serialization.

        Returns:
            A dictionary representation of the schedule
        """
        return {
            "id": self.id,
            "name": self.name,
            "task": self.task.to_dict(),
            "trigger_type": self.trigger_type,
            "trigger_args": self.trigger_args,
            "max_runs": self.max_runs,
            "created_at": self.created_at.isoformat() if self.created_at else None,
            "last_run": self.last_run.isoformat() if self.last_run else None,
            "run_count": self.run_count,
            "paused": self.paused,
            "execution_history": [
                {
                    "id": record["id"],
                    "start_time": record["start_time"].isoformat() if record.get("start_time") else None,
                    "end_time": record["end_time"].isoformat() if record.get("end_time") else None,
                    "success": record["success"],
                    "error": record["error"],
                    "duration": record["duration"]
                }
                for record in self.execution_history[-10:]  # Only include the last 10 records
            ]
        }

    @classmethod
    def from_dict(cls, data: Dict[str, Any], task_factory) -> 'Schedule':
        """
        Create a schedule from a dictionary representation.

        Args:
            data: Dictionary containing schedule data
            task_factory: Function to create a task from a dictionary

        Returns:
            A new Schedule instance
        """
        schedule = cls(
            task=task_factory(data["task"]),
            trigger_type=data["trigger_type"],
            trigger_args=data["trigger_args"],
            name=data.get("name"),
            max_runs=data.get("max_runs")
        )

        schedule.id = data["id"]
        schedule.created_at = datetime.fromisoformat(data["created_at"]) if data.get("created_at") else None
        schedule.last_run = datetime.fromisoformat(data["last_run"]) if data.get("last_run") else None
        schedule.run_count = data.get("run_count", 0)
        schedule.paused = data.get("paused", False)

        # We don't restore execution history to save memory

        return schedule



"""
Stylesheet manager for the ZScheduler application.
"""

from PyQt6.QtWidgets import QApplication


class StylesheetManager:
    """
    Manages application stylesheets and themes.
    """

    @staticmethod
    def apply_theme():
        """Apply the light green background with dark blue text theme."""
        stylesheet = """
        /* Global Application Style */
        QWidget {
            background-color: #e6ffe6;  /* Light green background */
            color: #00008B;             /* Dark blue text */
            font-family: 'Segoe UI', Arial, sans-serif;
        }

        /* Main Window */
        QMainWindow {
            background-color: #e6ffe6;
        }

        /* Toolbar */
        QToolBar {
            background-color: #d1ffd1;
            border-bottom: 1px solid #b3ffb3;
            spacing: 3px;
            padding: 3px;
        }

        QToolButton {
            background-color: #d1ffd1;
            border: 1px solid transparent;
            border-radius: 4px;
            padding: 5px;
        }

        QToolButton:hover {
            background-color: #b3ffb3;
            border: 1px solid #99ff99;
        }

        QToolButton:pressed {
            background-color: #99ff99;
        }

        /* Menu Bar */
        QMenuBar {
            background-color: #d1ffd1;
            color: #00008B;
        }

        QMenuBar::item {
            background-color: transparent;
            padding: 4px 8px;
        }

        QMenuBar::item:selected {
            background-color: #b3ffb3;
        }

        QMenu {
            background-color: #e6ffe6;
            border: 1px solid #b3ffb3;
        }

        QMenu::item {
            padding: 6px 25px 6px 20px;
        }

        QMenu::item:selected {
            background-color: #b3ffb3;
        }

        /* Tab Widget */
        QTabWidget::pane {
            border: 1px solid #b3ffb3;
            background-color: #e6ffe6;
        }

        QTabBar::tab {
            background-color: #d1ffd1;
            color: #00008B;
            border: 1px solid #b3ffb3;
            border-bottom: none;
            padding: 6px 12px;
            margin-right: 2px;
            border-top-left-radius: 4px;
            border-top-right-radius: 4px;
        }

        QTabBar::tab:selected {
            background-color: #e6ffe6;
            border-bottom: 1px solid #e6ffe6;
        }

        QTabBar::tab:!selected {
            margin-top: 2px;
        }

        /* Tree Widget */
        QTreeWidget {
            background-color: #e6ffe6;
            border: 1px solid #b3ffb3;
        }

        QTreeWidget::item {
            padding: 4px;
        }

        QTreeWidget::item:selected {
            background-color: #b3ffb3;
        }

        /* Buttons */
        QPushButton {
            background-color: #d1ffd1;
            color: #00008B;
            border: 1px solid #b3ffb3;
            border-radius: 4px;
            padding: 6px 12px;
        }

        QPushButton:hover {
            background-color: #b3ffb3;
        }

        QPushButton:pressed {
            background-color: #99ff99;
        }

        /* Text Displays */
        QTextEdit {
            background-color: #f0fff0;
            color: #00008B;
            border: 1px solid #b3ffb3;
        }

        /* Status Bar */
        QStatusBar {
            background-color: #d1ffd1;
            color: #00008B;
        }

        /* Frame Headers */
        QFrame {
            background-color: #e6ffe6;
        }

        QFrame#header_frame {
            background-color: #d1ffd1;
        }

        QLabel#header_label {
            color: #00008B;
            font-weight: bold;
            font-size: 14px;
        }

        /* Splitter */
        QSplitter::handle {
            background-color: #b3ffb3;
        }

        /* Form Elements */
        QLineEdit, QComboBox, QSpinBox {
            background-color: #f0fff0;
            border: 1px solid #b3ffb3;
            border-radius: 4px;
            padding: 4px;
            color: #00008B;
        }

        QLineEdit:focus, QComboBox:focus, QSpinBox:focus {
            border: 1px solid #99ff99;
        }

        /* Scrollbars */
        QScrollBar:vertical {
            background-color: #e6ffe6;
            width: 12px;
            margin: 12px 0 12px 0;
        }

        QScrollBar::handle:vertical {
            background-color: #b3ffb3;
            min-height: 20px;
            border-radius: 6px;
        }

        QScrollBar::add-line:vertical, QScrollBar::sub-line:vertical {
            background: none;
        }

        QScrollBar:horizontal {
            background-color: #e6ffe6;
            height: 12px;
            margin: 0 12px 0 12px;
        }

        QScrollBar::handle:horizontal {
            background-color: #b3ffb3;
            min-width: 20px;
            border-radius: 6px;
        }

        QScrollBar::add-line:horizontal, QScrollBar::sub-line:horizontal {
            background: none;
        }
        """

        QApplication.instance().setStyleSheet(stylesheet)



"""
Base Task class for ZScheduler
"""

import uuid
from abc import ABC, abstractmethod
from typing import Any, Dict, Optional

class Task(ABC):
    """
    Abstract base class for all task types.

    Tasks represent the actual work to be done when a schedule is triggered.
    Different task types handle different kinds of work (commands, Python functions, etc.).
    """

    def __init__(self, name: Optional[str] = None):
        """
        Initialize a new task.

        Args:
            name: Optional name for the task
        """
        self.id = str(uuid.uuid4())
        self.name = name

    @abstractmethod
    def execute(self) -> Any:
        """
        Execute the task.

        This method is called when the schedule is triggered.

        Returns:
            The result of the task execution
        """
        pass

    @abstractmethod
    def to_dict(self) -> Dict[str, Any]:
        """
        Convert the task to a dictionary for serialization.

        Returns:
            A dictionary representation of the task
        """
        return {
            "id": self.id,
            "name": self.name,
            "type": self.__class__.__name__
        }

    @classmethod
    @abstractmethod
    def from_dict(cls, data: Dict[str, Any]) -> 'Task':
        """
        Create a task from a dictionary representation.

        Args:
            data: Dictionary containing task data

        Returns:
            A new Task instance
        """
        pass



"""
Icon resources for ZScheduler application
"""

import os
import sys
import base64
import tempfile
from pathlib import Path

from PyQt6.QtGui import QIcon, QPixmap, QColor, QPainter, QFont, QPen, QBrush
from PyQt6.QtCore import QSize, Qt


# Base64 encoded icon data - created with a simple clock icon
ICON_DARK = """
iVBORw0KGgoAAAANSUhEUgAAAEAAAABACAYAAACqaXHeAAAACXBIWXMAAA7EAAAOxAGVKw4bAAAGg0lEQVR4nO2ba2wUVRTHf7O7
bXc7u9tSaAWkCBQV0QgGQyUSjB+UQHjEB0Y0PiLGqDE+0PiIH4zGDzVRYzQxPqLGRI1G1KgxGhOMiSIYFdACRqC8SgstLftou7sz
x/yzs9Pdx+zur+1Dpif5Z2bvnDn3P/fcc+65cwNMYQpTmEIOoZNlaDBq9QFLgAZgFpAE4sCw87sE+N0GdumwA2jPVUddu+5YwypD
9kNAIzAbeAqYD8wFZgC1HDOo+DhsMKEXiADdQAfwC7AX+BnYrUN7/rFk5sor1105X428NuB+4Hpgdr46FgqEchuQANqAvUCnK3CS
I6ca4DARhjhR5JD/8LjrG+BeHfb4WQs02GrNEQXUaCNwO/Ba0SWfZplS5haP60cY4jZyKcRIcJGmw3pgQUm0l4VKK2CxDk8Cdydu
zreHZl+8fGXmykXI1Aq9FCB7eWagzcAGoAmYhsQlXwOfAQ0KB1JHvwYgBbwKTKZXvwq4BnjXXQgSsMIGm3SoBn6i6OFHwUSmwEId
4sAbQJ1h0qwL+jBMOWDZAn+aBl1LCHX9ZpyjA4KXQwdwN3AHiRFvA9YDt+hwOdCDzKPjkGsSVMBa4OLYsQhYgoyRGF4kYSLmrwzQ
qsM7wJrBS2/8cPrDHTUeJJaKM0ei0jgJHLM2QZxjSwxYpsODwLV+1CAI15hEAlM1yOfPIIFlBDhZfE4mQqqAmcDtRQszjaJiyqtb
CDL6JcBFwD4kKgsZZMp5vwPcGMbRWSihXGvjwOfIRh3GWC/M2gNR4C8kQRkRueWAm8CfYSaEUngt0y4wHXgBWJalrmpIAj8AHwBv
ozgQZAQovJhqgCuA+sJlOjVQrnlfbdvYtlE5ZMJhQjI9ZcNQ25Ot3wgUwmeYGWR3T7pnVDoHxP2oIdxiKDojrktuX3WvfQNIus/S
aQeJ6wp9RJI9UfT3jymGD1IAyw7gbyTGZzCosUHTwEiYhi4PnzHb2ltRHBJPzNpVNuw5cn/8gyOPrC2BgCcdyeA0IkiQS+hCG4VA
lhrwQDJ+ZJ4+q11Tn39rnf77/EVlFCvnKZA3GhRom04iBHXAk5QooxsJIRdEA5bGvx354rkG+5zWWjVn4WfVm/UqXkRs+1Y7zpfd
dQtvsy/IWUFQCu4ErNONZqDBABvwWP9B2Krf5dd2mdqC//ToeZ84Smg27bgBWJsQOH6VXRAMqdDfTgN5nPrw4zrchGSOiykhE4JE
uNLy+DPAA31fy8ZpO7Lm7jUOri909dusQBC/PcBUqQiTz1/lHn5NQxK7hQbTVODP4h7iiDvQ8akWbe5tmqmtQCfdiq7KFYTCJUB5
9wAdfhh+j3hgLdbiMPCGUHZcEeJdIXoeCS9+uwQ7TXNFEhh2KcBVwHJkE9SB6YyYUzMUy83u/1cBz/v4PDCJ+VtQQQOeJI+/O7tc
KpgA/KqJU8A7oOlIPGhGIqpvoEkAK5HIbwGzIKApYFjYlpG9BygEuWJ4utGKI7GgKpMDAF0FSzu+rrGM6SgDgwDYYVTrKqP/dyB1
Pzr8QVY+6xL3RhwMQX9PJiostS3TU9jxleIfRVOIyzMBkqHUxZD+8vC0qvVerI+2SgUtyDJXdHgpYAQ4mM+fuZD3aZAO24B1gFmt
V/e11n54akGEuEQiHiVDM7oF9HbPNr+z4+2RO9Y/Opb6MnyKLJgZVpUF59/lkohjxO91MvpBp1rHA9U1rcBjQDNwyHB2+hFkroZB
HbLpGUBouV26z28dsSSe+wjoQ8ogKxw5/eiFFWCDpcPHSPSdQZY9+9cQJJJcAmwELiP8ntCDlMJHnAJLPg5lUYAOOwg+gsgkAm3A
w8hDZIPgLKRsdCbhS+itwB/OvyCU6zzgSdBrHd9nEYNTtHY4eQR4EdkzDBQwU3HPPP2UsA34H9l6jwGHkBrDnZxcxZsOGlI2i+Ep
xKSUVvuAF4CXkRFvQmJ8GpMU4eQU5kPPEPCFDq8Dq5DDVjefKKqQ5bY9T//5OCEsFtXA504l+iLgCaBVGaYF1NqYaVV1MDFRnlSg
30uamA6NyLlDA/K0qsEpi08D9snehWIeQVcBHwEXILnFy9jEeiP/niKxSOUph6lJ+NoFg+iQAto1WTqbkZDZBxw+WL137aLwXZ3Q
F1kcXevkGEHrQBE47XMBn4xSkny5Sw5zCpvFn8IUpjCF04//AQ32sOjiGTjqAAAAAElFTkSuQmCC
"""

ICON_LIGHT = """
iVBORw0KGgoAAAANSUhEUgAAAEAAAABACAYAAACqaXHeAAAACXBIWXMAAA7EAAAOxAGVKw4bAAAGdklEQVR4nO2bW2wUVRjHf9vd
bbvdnW3prdBSLqVYxBtBDQnGqNEHJRATjfHBGI3G+01N1OiDRo0v+qAPaqJG40ONCY8aI/FSMUYSBEGBAi1QbKHQG7vb3Z3j+5+Z
nel2Z3Zndq5mTvJldt/znXP+5zvfba1AHHHEEceFhMafza3AEmARUA/kAVkgARxzfpuAP2xgmw7bm7GtvnQ2aFvsNeReBcwB5gD3
AzOB6UANaamIN+0UMgO9QCfQAfwC7AP+BL7ToT2XyJy96oabruqqltMGvArcDUzPl49coZBsA7JAG7AX6HSJcPqoqQKHSdDPyaS6
3PbLCLe9CbymAIuAm4CHgCpqiNkDHAZ6UDpQ3Nx+RaJ+Hf4BtumwsfmXZ+P+uiDh1d8APEXxQxUZ3LokP0pJYtrvYeAR4CF/XwqY
r0Mj8JKisFLlHFMJDA26p0jQHyYqDwW8BDzg76tUGw94s7lV/7b67U2NDd1pEtTrEl+5FHCg8bhqBxP0a6i+K9xnfo/YZcOt9v9p
UMlko+XRwa5xSk6214SL7mqbUo+EZEJC+XlIqd7smcmjwIumTaVagAWcRGLpdiAlyrjEgMvCVrxx/HnuGV5drjxQ7MTGWWv42jr5
3OfXAO8A38067qtEf98XwZY6euR03whHnQa8BKwArlVIaxUDuMqfVkomnYXyzhgGGc4ZwBvAjZTXRC5JGgPVGONdGOQ4NzGgCYEK
SkqkGKKNRX3OPgrXuJ9ivSMLcXGaUjXNl8PAnqCBZIEObjyjT/ylMOI95H9xUH4LuAu406/sRcabInc70CVoM6PkZ8SbFlBRk5qf
XdL9e6ONZCG+OOVxx5IPSCD7LMZ7O9iKtAWA3QU5ZqPZ+Yb9wYbEWU/7g9D1T4HLkUB6BNiH7O9uJCEKQgrhVSFYIvyBQUYpJ9eA
68JWpCqO8tXu1WfzdiWCS+B+z3z33OMN4yTGF2A2Nw9amAQ5y5neTXa36+mFjlf0FsaXFi4wXH8DvtfhDM54Ngl6TRZUDnFhgGwc
22h7su3JCUdra5XU3OjdJ9O37zCLzG58wVvAHj9jbrjZeDSUe02+pcfTVz1id7ii575N2j//PKphxMhHMy5crsU5emyIXmVCm0cq
nEWyump7QQVIC1hJwrZnRj/+aOn4S9tkDsuVneoK/ar6pY0eQa8hJuqLENcPiLsSsBKf8UamCbgJaLLBttFDXzrZvBW4whW8loaW
18epX9ymsGia6QgBbCKWi/LxHIKx7eJFechfha5HKsIgw1TwTx1uS2Te/MXCnIC8T9wOQ9rcCNSa+bgQ2DvjZ37lwDBQTBrPB/7M
c7v8Q5F9XCusHHaJJZgrQCWOUBtudvcdVqx3ttFXmu9Bz4iN1gAcNXFMIb2GF6dyOaDhj3oDrGDvxWl2oTTZCm7fPaDDXnP2d3G5
9Z4VSXJjCTIKdksppCuVINdBNo/jE5MaXQWoNX1DcRJZ/Y9FQaWahEoy/Y2KB7hKUH0e9nny5GTCFBgTWAESHiAbwAWYvp+LLYO5
SYvJ1KCsOqBwJeS9G6vTp9k1j20+CvxxKZKEJRQgt/snoPZbxg+4aaQhEmiu2UE5GIAiuB6dw4i/f0IBnNB2xsIMVz0bY5i5idMK
KczmAIPEcQRR6Z72dQG1ueqGbZYxRXy0wM9KDrtCZBQwjNQCoclQAeorruS3ccXxPPDkAlGgvIHR9n3WY+S8tTC88S3Ie/QPKLpC
yDoJ7mTcnd2FeGPjw2olKuBf4C3gE+SIy5S8FcJ9MSrh9HOh6QZazs99EGgF1gILGCmAfdHOtVxbp9Q14Nj70eyM7FtvBvM1ZAGD
E4GeSyTzEmxEzpvrDHIKzP9ESPkcH8+NM5+ZmA3dwHvA+8DPQP957Uv8W4nD7kQOYG9EzqeHkBp/EAkDvw7x3oyAPRrcr8NDSL6Q
RjI2G0mo7OeP9qNCWaGw2Um5YQCp3H5GkqknkSzOjQkcQcquPvUqWSXl7oUWcFCHj4GNwGLkpmgaSSUbkKzvcAnZ9B3ElaQ6cyOd
icgGKZNb0GvIUpePqJuSAn5y3udhJHE6nlDp80AHuMQvXYETqF9G/H2Q9xRpJ2jfCezdZ89aEz5qL8FL0eXIOX+YsGFnFGhrAnOr
GG4q5eV8YxpiwsVxEN9XDCe0/vaCXNscRxxx/H/xHxHxsZZ5XRG5AAAAAElFTkSuQmCC
"""


class IconProvider:
    """
    Provides application icons in various formats and themes
    """

    _instance = None
    _icons = {}

    @staticmethod
    def get_instance():
        """Get the singleton instance of IconProvider"""
        if IconProvider._instance is None:
            IconProvider._instance = IconProvider()
        return IconProvider._instance

    def __init__(self):
        """Initialize the icon provider"""
        if IconProvider._instance is not None:
            raise RuntimeError("IconProvider is a singleton. Use get_instance() instead.")

        # Create icons from base64 data
        self._icons['app_dark'] = self._create_icon_from_base64(ICON_DARK)
        self._icons['app_light'] = self._create_icon_from_base64(ICON_LIGHT)

    def _create_icon_from_base64(self, base64_data):
        """
        Create a QIcon from base64 encoded data

        Args:
            base64_data: The base64 encoded image data

        Returns:
            QIcon object created from the data
        """
        try:
            # Clean up the base64 data - remove whitespace and newlines
            clean_data = ''.join(base64_data.strip().split())

            # Create a pixmap from the base64 data
            pixmap = QPixmap()
            if not pixmap.loadFromData(base64.b64decode(clean_data)):
                # If loading fails, create a simple colored square as fallback
                pixmap = QPixmap(64, 64)
                pixmap.fill(QColor("#50fa7b"))

            # Create an icon from the pixmap
            return QIcon(pixmap)
        except Exception as e:
            # Log the error and return a fallback icon
            print(f"Error creating icon: {e}")
            pixmap = QPixmap(64, 64)
            pixmap.fill(QColor("#50fa7b"))
            return QIcon(pixmap)

    def get_app_icon(self, theme='dark'):
        """
        Get the application icon for the specified theme

        Args:
            theme: 'dark' or 'light'

        Returns:
            QIcon object for the application
        """
        key = 'app_dark' if theme == 'dark' else 'app_light'
        return self._icons[key]

    def set_window_icon(self, window, theme='dark'):
        """
        Set the icon for a window

        Args:
            window: The window to set the icon for
            theme: 'dark' or 'light'
        """
        window.setWindowIcon(self.get_app_icon(theme))

    def save_system_tray_icon(self, theme='dark'):
        """
        Save the icon to a temporary file for system tray use

        Args:
            theme: 'dark' or 'light'

        Returns:
            Path to the temporary icon file
        """
        # Create a temporary file
        fd, path = tempfile.mkstemp(suffix='.png')
        os.close(fd)

        # Save the icon as PNG
        key = 'app_dark' if theme == 'dark' else 'app_light'
        self._icons[key].pixmap(48, 48).save(path, 'PNG')

        return path

class IconManager:
    """
    Manages application icons based on the style guide.
    """

    # Icon sizes
    SMALL = QSize(16, 16)
    MEDIUM = QSize(24, 24)
    LARGE = QSize(32, 32)

    @staticmethod
    def get_icon_path(icon_name):
        """Get the path to an icon file."""
        # In a real implementation, this would look for SVG files
        # For now, we'll use text-based icons
        return None

    @staticmethod
    def get_icon(icon_name):
        """Get a QIcon for the specified icon name."""
        path = IconManager.get_icon_path(icon_name)
        if path and os.path.exists(path):
            return QIcon(path)
        else:
            # Return a text icon
            return IconManager.create_text_icon(icon_name[0].upper() if icon_name else "?")

    @staticmethod
    def create_text_icon(text, size=SMALL, bg_color="#d1ffd1", text_color="#00008B"):
        """Create a text-based icon when an image is not available."""
        # Create a pixmap
        pixmap = QPixmap(size)
        pixmap.fill(Qt.GlobalColor.transparent)

        # Create a painter and draw the text
        painter = QPainter(pixmap)
        painter.setRenderHint(QPainter.RenderHint.Antialiasing)

        # Draw background circle
        painter.setPen(QPen(QColor(bg_color)))
        painter.setBrush(QBrush(QColor(bg_color)))
        painter.drawEllipse(0, 0, size.width(), size.height())

        # Draw text
        font = QFont("Arial", size.width() // 2)
        font.setBold(True)
        painter.setFont(font)
        painter.setPen(QPen(QColor(text_color)))
        painter.drawText(pixmap.rect(), Qt.AlignmentFlag.AlignCenter, text)

        painter.end()

        return QIcon(pixmap)

    # Define methods for common icons
    @staticmethod
    def new_icon():
        return IconManager.create_text_icon("N")

    @staticmethod
    def edit_icon():
        return IconManager.create_text_icon("E")

    @staticmethod
    def delete_icon():
        return IconManager.create_text_icon("D")

    @staticmethod
    def start_icon():
        return IconManager.create_text_icon("S")

    @staticmethod
    def stop_icon():
        return IconManager.create_text_icon("X")

    @staticmethod
    def pause_icon():
        return IconManager.create_text_icon("P")

    @staticmethod
    def resume_icon():
        return IconManager.create_text_icon("R")

    @staticmethod
    def run_now_icon():
        return IconManager.create_text_icon("►")

    @staticmethod
    def settings_icon():
        return IconManager.create_text_icon("⚙")

    @staticmethod
    def about_icon():
        return IconManager.create_text_icon("?")



"""
Schedule list view component for ZScheduler
"""

import logging
from typing import List, Optional

from PyQt6.QtWidgets import (
    QWidget, QVBoxLayout, QTableView, QHeaderView, QAbstractItemView,
    QToolBar, QLabel
)
from PyQt6.QtCore import Qt, QAbstractTableModel, QModelIndex, QVariant, pyqtSlot
from PyQt6.QtGui import QColor

from zscheduler_lib.zscheduler import Scheduler
from zscheduler_lib.zscheduler.core.schedule import Schedule

logger = logging.getLogger(__name__)

class ScheduleTableModel(QAbstractTableModel):
    """Table model for displaying schedules in a table view."""

    COLUMNS = ["Name", "Type", "Next Run", "Status", "Runs", "Last Run"]

    def __init__(self, parent=None):
        """Initialize the model."""
        super().__init__(parent)
        self.schedules = []

    def rowCount(self, parent=QModelIndex()) -> int:
        """Return the number of rows."""
        return len(self.schedules)

    def columnCount(self, parent=QModelIndex()) -> int:
        """Return the number of columns."""
        return len(self.COLUMNS)

    def data(self, index: QModelIndex, role=Qt.ItemDataRole.DisplayRole):
        """Return the data at the given index."""
        if not index.isValid():
            return QVariant()

        if index.row() >= len(self.schedules) or index.row() < 0:
            return QVariant()

        schedule = self.schedules[index.row()]

        if role == Qt.ItemDataRole.DisplayRole:
            if index.column() == 0:  # Name
                return schedule.get('name') or f"Task-{schedule.get('id', '')[:8]}"
            elif index.column() == 1:  # Type
                task = schedule.get('task')
                return task.__class__.__name__ if task else ''
            elif index.column() == 2:  # Next Run
                return str(schedule.get('next_run')) if schedule.get('next_run') else "Not scheduled"
            elif index.column() == 3:  # Status
                return "Paused" if schedule.get('paused', False) else "Active"
            elif index.column() == 4:  # Runs
                return str(schedule.get('run_count'))
            elif index.column() == 5:  # Last Run
                return str(schedule.get('last_run')) if schedule.get('last_run') else "Never"

        elif role == Qt.ItemDataRole.BackgroundRole:
            # Highlight paused schedules
            if schedule.get('paused', False) and index.column() == 3:
                return QColor(255, 200, 200)

        return QVariant()

    def headerData(self, section: int, orientation: Qt.Orientation, role=Qt.ItemDataRole.DisplayRole):
        """Return the header data."""
        if orientation == Qt.Orientation.Horizontal and role == Qt.ItemDataRole.DisplayRole:
            return self.COLUMNS[section]

        return QVariant()

    def update_schedules(self, schedules: List[Schedule]) -> None:
        """Update the model with new schedules."""
        self.beginResetModel()
        self.schedules = schedules
        self.endResetModel()

class ScheduleListView(QWidget):
    """Widget for displaying schedules in a table view."""

    def __init__(self, scheduler: Scheduler, parent=None):
        """Initialize the view."""
        super().__init__(parent)
        self.scheduler = scheduler

        # Create layout
        layout = QVBoxLayout(self)

        # Create table view
        self.table_view = QTableView()
        self.table_model = ScheduleTableModel()
        self.table_view.setModel(self.table_model)

        # Configure table view
        self.table_view.setSelectionBehavior(QAbstractItemView.SelectionBehavior.SelectRows)
        self.table_view.setSelectionMode(QAbstractItemView.SelectionMode.SingleSelection)
        self.table_view.setAlternatingRowColors(True)
        self.table_view.setSortingEnabled(True)
        self.table_view.horizontalHeader().setSectionResizeMode(QHeaderView.ResizeMode.Stretch)

        # Add to layout
        layout.addWidget(self.table_view)

        # Set layout
        self.setLayout(layout)

    def update_schedules(self, schedules: List[Schedule]) -> None:
        """Update the view with new schedules."""
        self.table_model.update_schedules(schedules)

    def get_selected_schedule(self) -> Optional[Schedule]:
        """Return the currently selected schedule."""
        indexes = self.table_view.selectionModel().selectedIndexes()
        if not indexes:
            return None

        row = indexes[0].row()
        return self.table_model.schedules[row] if 0 <= row < len(self.table_model.schedules) else None



"""
Dialog for creating and editing schedules
"""

import logging
from datetime import datetime, timedelta

from PyQt6.QtWidgets import (
    QDialog, QVBoxLayout, QHBoxLayout, QFormLayout,
    QLabel, QLineEdit, QComboBox, QSpinBox, QDateTimeEdit,
    QPushButton, QDialogButtonBox, QTabWidget, QWidget,
    QTextEdit, QCheckBox, QGroupBox
)
from PyQt6.QtCore import Qt, QDateTime
from PyQt6.QtGui import QIcon

from zscheduler_app.scheduler.scheduler import Scheduler, Schedule

logger = logging.getLogger(__name__)

class NewScheduleDialog(QDialog):
    """
    Dialog for creating or editing a schedule
    """

    def __init__(self, scheduler, parent=None, schedule=None):
        """
        Initialize the dialog

        Args:
            scheduler: The scheduler instance
            parent: Parent widget
            schedule: Schedule to edit, or None for new schedule
        """
        super().__init__(parent)
        self.scheduler = scheduler
        self.schedule = schedule
        self.edit_mode = schedule is not None

        # Set window properties
        self.setWindowTitle(f"{'Edit' if self.edit_mode else 'New'} Schedule")
        self.resize(500, 400)

        # Set up UI
        self._setup_ui()

        # Fill in existing data if editing
        if self.edit_mode:
            self._populate_form()

    def _setup_ui(self):
        """Set up the dialog UI"""
        main_layout = QVBoxLayout(self)

        # Form layout for basic info
        form_layout = QFormLayout()

        # Name field
        self.name_edit = QLineEdit()
        form_layout.addRow("Name:", self.name_edit)

        # Command field
        self.command_edit = QLineEdit()
        form_layout.addRow("Command:", self.command_edit)

        # Schedule type
        self.type_combo = QComboBox()
        self.type_combo.addItems(["Interval", "Cron", "Once"])
        self.type_combo.currentIndexChanged.connect(self._on_type_changed)
        form_layout.addRow("Schedule Type:", self.type_combo)

        # Add form to main layout
        main_layout.addLayout(form_layout)

        # Tab widget for schedule type specific settings
        self.tabs = QTabWidget()

        # Interval tab
        interval_widget = QWidget()
        interval_layout = QVBoxLayout(interval_widget)

        interval_form = QFormLayout()

        # Interval inputs
        self.interval_seconds = QSpinBox()
        self.interval_seconds.setRange(0, 59)
        self.interval_seconds.setSuffix(" seconds")

        self.interval_minutes = QSpinBox()
        self.interval_minutes.setRange(0, 59)
        self.interval_minutes.setSuffix(" minutes")

        self.interval_hours = QSpinBox()
        self.interval_hours.setRange(0, 23)
        self.interval_hours.setSuffix(" hours")

        self.interval_days = QSpinBox()
        self.interval_days.setRange(0, 365)
        self.interval_days.setSuffix(" days")

        # Add interval inputs to form
        interval_form.addRow("Days:", self.interval_days)
        interval_form.addRow("Hours:", self.interval_hours)
        interval_form.addRow("Minutes:", self.interval_minutes)
        interval_form.addRow("Seconds:", self.interval_seconds)

        # Set default interval
        self.interval_minutes.setValue(5)

        interval_layout.addLayout(interval_form)
        self.tabs.addTab(interval_widget, "Interval")

        # Cron tab
        cron_widget = QWidget()
        cron_layout = QVBoxLayout(cron_widget)

        cron_layout.addWidget(QLabel("Cron Expression:"))
        self.cron_edit = QLineEdit()
        self.cron_edit.setPlaceholderText("* * * * *")
        cron_layout.addWidget(self.cron_edit)

        cron_layout.addWidget(QLabel("Format: minute hour day_of_month month day_of_week"))
        cron_layout.addWidget(QLabel("Example: '*/5 * * * *' runs every 5 minutes"))

        self.tabs.addTab(cron_widget, "Cron")

        # Once tab
        once_widget = QWidget()
        once_layout = QVBoxLayout(once_widget)

        once_layout.addWidget(QLabel("Run Once At:"))
        self.datetime_edit = QDateTimeEdit()
        self.datetime_edit.setDateTime(QDateTime.currentDateTime().addSecs(300))  # Default to 5 mins from now
        self.datetime_edit.setCalendarPopup(True)
        once_layout.addWidget(self.datetime_edit)

        self.tabs.addTab(once_widget, "Once")

        # Add tabs to main layout
        main_layout.addWidget(self.tabs)

        # Additional options
        options_group = QGroupBox("Options")
        options_layout = QVBoxLayout(options_group)

        self.enabled_check = QCheckBox("Enabled")
        self.enabled_check.setChecked(True)
        options_layout.addWidget(self.enabled_check)

        main_layout.addWidget(options_group)

        # Buttons
        button_box = QDialogButtonBox(QDialogButtonBox.StandardButton.Ok | QDialogButtonBox.StandardButton.Cancel)
        button_box.accepted.connect(self.accept)
        button_box.rejected.connect(self.reject)
        main_layout.addWidget(button_box)

        # Set active tab based on initial type selection
        self._on_type_changed(0)

    def _on_type_changed(self, index):
        """Handle schedule type change"""
        self.tabs.setCurrentIndex(index)

    def _populate_form(self):
        """Populate the form with existing schedule data"""
        schedule_data = self.schedule.to_dict()

        # Set basic fields
        self.name_edit.setText(schedule_data.get("name", ""))
        self.command_edit.setText(schedule_data.get("command", ""))
        self.enabled_check.setChecked(schedule_data.get("enabled", True))

        # Set schedule type and related fields
        schedule_type = schedule_data.get("schedule_type", "interval")
        if schedule_type == "interval":
            self.type_combo.setCurrentIndex(0)

            # Set interval values
            interval = schedule_data.get("interval", 0)
            days = interval // 86400
            remainder = interval % 86400
            hours = remainder // 3600
            remainder = remainder % 3600
            minutes = remainder // 60
            seconds = remainder % 60

            self.interval_days.setValue(days)
            self.interval_hours.setValue(hours)
            self.interval_minutes.setValue(minutes)
            self.interval_seconds.setValue(seconds)

        elif schedule_type == "cron":
            self.type_combo.setCurrentIndex(1)
            self.cron_edit.setText(schedule_data.get("cron_expression", ""))

        elif schedule_type == "once":
            self.type_combo.setCurrentIndex(2)
            try:
                dt = datetime.fromisoformat(schedule_data.get("cron_expression", ""))
                self.datetime_edit.setDateTime(QDateTime(dt.year, dt.month, dt.day, dt.hour, dt.minute, dt.second))
            except (ValueError, TypeError):
                # If parsing fails, use current time + 5 minutes
                self.datetime_edit.setDateTime(QDateTime.currentDateTime().addSecs(300))

    def accept(self):
        """Handle dialog acceptance"""
        # Get data from form
        name = self.name_edit.text().strip()
        command = self.command_edit.text().strip()
        schedule_type = ["interval", "cron", "once"][self.type_combo.currentIndex()]
        enabled = self.enabled_check.isChecked()

        # Validate inputs
        if not name:
            name = "Unnamed Schedule"

        if not command:
            logger.warning("No command specified")
            command = "echo 'No command specified'"

        # Prepare schedule data
        schedule_data = {
            "name": name,
            "command": command,
            "schedule_type": schedule_type,
            "enabled": enabled,
        }

        # Get type-specific data
        if schedule_type == "interval":
            interval = (self.interval_days.value() * 86400 +
                       self.interval_hours.value() * 3600 +
                       self.interval_minutes.value() * 60 +
                       self.interval_seconds.value())

            if interval == 0:
                interval = 60  # Default to 1 minute

            schedule_data["interval"] = interval

        elif schedule_type == "cron":
            cron_expr = self.cron_edit.text().strip()
            if not cron_expr:
                cron_expr = "*/5 * * * *"  # Default to every 5 minutes

            schedule_data["cron_expression"] = cron_expr

        elif schedule_type == "once":
            dt = self.datetime_edit.dateTime().toPython()
            schedule_data["cron_expression"] = dt.isoformat()

        # Update or create schedule
        try:
            if self.edit_mode:
                self.scheduler.update_schedule(self.schedule.id, schedule_data)
                logger.info(f"Updated schedule: {name}")
            else:
                schedule_id = self.scheduler.add_schedule(schedule_data)
                logger.info(f"Created schedule: {name} (ID: {schedule_id})")

            # Accept dialog
            super().accept()

        except Exception as e:
            logger.error(f"Error saving schedule: {e}")
            # Keep dialog open



"""
Core scheduler implementation for ZScheduler
"""

import logging
import threading
import time
from typing import Any, Dict, List, Optional, Union, Callable

from apscheduler.schedulers.background import BackgroundScheduler
from apscheduler.jobstores.memory import MemoryJobStore

from ..tasks.task import Task
from ..events.event_emitter import EventEmitter
from .schedule_builder import ScheduleBuilder
from .schedule import Schedule

logger = logging.getLogger(__name__)

class Scheduler:
    """
    The main scheduler class that manages all scheduled tasks.

    Features:
    - Schedule OS commands and Python functions/classes/modules
    - One-time, recurring, and N-times schedules
    - Flexible time specifications (absolute dates, relative times, cron expressions)
    - Event-based architecture for monitoring and notifications
    """

    def __init__(self, job_store: Optional[Any] = None):
        """
        Initialize a new scheduler instance.

        Args:
            job_store: Optional custom job store, defaults to memory store
        """
        self._scheduler = BackgroundScheduler(
            jobstores={'default': job_store or MemoryJobStore()}
        )
        self._events = EventEmitter()
        self._schedules: Dict[str, Schedule] = {}
        self._running = False

    def schedule(self, target: Union[str, Callable, Any], *args, **kwargs) -> ScheduleBuilder:
        """
        Creates a schedule for the given target.

        Args:
            target: The target to schedule (command string, function, etc.)
            *args: Positional arguments to pass to the target function
            **kwargs: Keyword arguments to pass to the target function

        Returns:
            A ScheduleBuilder to configure the schedule

        Examples:
            >>> scheduler.schedule("echo Hello").daily_at("12:00")
            >>> scheduler.schedule(my_func, arg1, arg2).every(minutes=5)
        """
        return ScheduleBuilder(self, target, *args, **kwargs)

    def _add_schedule(self, schedule: Schedule) -> str:
        """
        Add a schedule to the scheduler.

        Args:
            schedule: The schedule to add

        Returns:
            The schedule ID
        """
        self._schedules[schedule.id] = schedule

        # Add to APScheduler
        job = self._scheduler.add_job(
            schedule.execute,
            schedule.trigger_type,
            **schedule.trigger_args,
            id=schedule.id,
            name=schedule.name or f"Task-{schedule.id[:8]}"
        )

        self._events.emit("schedule_added", schedule)
        return schedule.id

    def remove_schedule(self, schedule_id: str) -> bool:
        """
        Remove a schedule from the scheduler.

        Args:
            schedule_id: ID of the schedule to remove

        Returns:
            True if removed, False if not found
        """
        if schedule_id not in self._schedules:
            return False

        schedule = self._schedules.pop(schedule_id)
        self._scheduler.remove_job(schedule_id)
        self._events.emit("schedule_removed", schedule)
        return True

    def get_schedule(self, schedule_id: str) -> Optional[Schedule]:
        """
        Get a schedule by its ID.

        Args:
            schedule_id: ID of the schedule to retrieve

        Returns:
            The schedule if found, None otherwise
        """
        return self._schedules.get(schedule_id)

    def get_schedules(self) -> List[Schedule]:
        """
        Get all schedules.

        Returns:
            List of all schedules
        """
        return list(self._schedules.values())

    def pause_schedule(self, schedule_id: str) -> bool:
        """
        Pause a schedule.

        Args:
            schedule_id: ID of the schedule to pause

        Returns:
            True if paused, False if not found
        """
        if schedule_id not in self._schedules:
            return False

        self._scheduler.pause_job(schedule_id)
        schedule = self._schedules[schedule_id]
        schedule.paused = True
        self._events.emit("schedule_paused", schedule)
        return True

    def resume_schedule(self, schedule_id: str) -> bool:
        """
        Resume a paused schedule.

        Args:
            schedule_id: ID of the schedule to resume

        Returns:
            True if resumed, False if not found
        """
        if schedule_id not in self._schedules:
            return False

        self._scheduler.resume_job(schedule_id)
        schedule = self._schedules[schedule_id]
        schedule.paused = False
        self._events.emit("schedule_resumed", schedule)
        return True

    def start(self) -> None:
        """Start the scheduler"""
        if self._running:
            return

        self._scheduler.start()
        self._running = True
        self._events.emit("scheduler_started", self)

    def stop(self) -> None:
        """Stop the scheduler"""
        if not self._running:
            return

        self._scheduler.shutdown()
        self._running = False
        self._events.emit("scheduler_stopped", self)

    def on(self, event_name: str, callback: Callable) -> None:
        """
        Register an event listener.

        Args:
            event_name: Name of the event to listen for
            callback: Function to call when the event occurs
        """
        self._events.on(event_name, callback)

    @property
    def running(self) -> bool:
        """Whether the scheduler is running"""
        return self._running



"""
About dialog for ZScheduler application
"""

import logging
from PyQt6.QtWidgets import (
    QDialog, QVBoxLayout, QLabel, QPushButton, QDialogButtonBox,
    QTabWidget, QWidget, QTextBrowser, QHBoxLayout
)
from PyQt6.QtCore import Qt
from PyQt6.QtGui import QPixmap, QFont

from zscheduler_app import __version__
from zscheduler_app.ui.icons import IconProvider

logger = logging.getLogger(__name__)

class AboutDialog(QDialog):
    """
    Dialog showing information about the application
    """

    def __init__(self, parent=None):
        """
        Initialize the about dialog

        Args:
            parent: Parent widget
        """
        super().__init__(parent)
        self.setWindowTitle("About ZScheduler")
        self.resize(500, 400)
        self.setModal(True)

        # Create UI
        self._setup_ui()

    def _setup_ui(self):
        """Set up the about UI"""
        layout = QVBoxLayout(self)

        # Header with app name and version
        header_layout = QHBoxLayout()

        # App icon
        icon_provider = IconProvider.get_instance()
        icon_label = QLabel()
        pixmap = icon_provider.get_app_icon().pixmap(64, 64)
        icon_label.setPixmap(pixmap)
        header_layout.addWidget(icon_label)

        # App name and version
        title_layout = QVBoxLayout()

        app_name = QLabel("ZScheduler")
        font = app_name.font()
        font.setPointSize(16)
        font.setBold(True)
        app_name.setFont(font)
        title_layout.addWidget(app_name)

        version = QLabel(f"Version {__version__}")
        title_layout.addWidget(version)

        header_layout.addLayout(title_layout)
        header_layout.addStretch()

        layout.addLayout(header_layout)

        # Tab widget for different sections
        tabs = QTabWidget()

        # About tab
        about_tab = QWidget()
        about_layout = QVBoxLayout(about_tab)

        about_text = QTextBrowser()
        about_text.setOpenExternalLinks(True)
        about_text.setHtml("""
        <p>ZScheduler is a high-performance task scheduling application with an intuitive user interface.</p>

        <p>Key features:</p>
        <ul>
            <li>Multiple visualization options: List, Calendar, and Timeline views</li>
            <li>Flexible scheduling: Interval-based, cron expressions, and one-time schedules</li>
            <li>Dark and light themes</li>
            <li>System tray integration</li>
            <li>Detailed statistics and insights</li>
        </ul>

        <p>Built with Python and PyQt6.</p>
        """)

        about_layout.addWidget(about_text)
        tabs.addTab(about_tab, "About")

        # License tab
        license_tab = QWidget()
        license_layout = QVBoxLayout(license_tab)

        license_text = QTextBrowser()
        license_text.setHtml("""
        <h3>MIT License</h3>

        <p>Copyright (c) 2024 ZScheduler Contributors</p>

        <p>Permission is hereby granted, free of charge, to any person obtaining a copy
        of this software and associated documentation files (the "Software"), to deal
        in the Software without restriction, including without limitation the rights
        to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
        copies of the Software, and to permit persons to whom the Software is
        furnished to do so, subject to the following conditions:</p>

        <p>The above copyright notice and this permission notice shall be included in all
        copies or substantial portions of the Software.</p>

        <p>THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
        IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
        FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
        AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
        LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
        OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
        SOFTWARE.</p>
        """)

        license_layout.addWidget(license_text)
        tabs.addTab(license_tab, "License")

        # Credits tab
        credits_tab = QWidget()
        credits_layout = QVBoxLayout(credits_tab)

        credits_text = QTextBrowser()
        credits_text.setHtml("""
        <h3>Credits</h3>

        <p>ZScheduler uses the following third-party libraries:</p>

        <ul>
            <li>PyQt6 - Licensed under GPL v3</li>
            <li>PyQtGraph - Licensed under MIT</li>
            <li>python-dateutil - Licensed under BSD-3-Clause</li>
        </ul>

        <p>Special thanks to all contributors and open source software that made this possible.</p>
        """)

        credits_layout.addWidget(credits_text)
        tabs.addTab(credits_tab, "Credits")

        layout.addWidget(tabs)

        # Button box
        button_box = QDialogButtonBox(QDialogButtonBox.StandardButton.Close)
        button_box.rejected.connect(self.reject)
        layout.addWidget(button_box)



#!/usr/bin/env python3
"""
ZScheduler Application Launcher

This script provides a simple way to launch the ZScheduler application.
"""

import sys
import os

# Add the current directory to the path
current_dir = os.path.dirname(os.path.abspath(__file__))
if current_dir not in sys.path:
    sys.path.insert(0, current_dir)

# Import and run the application
from zscheduler_app.main import main

if __name__ == "__main__":
    main()



"""
Event emitter for ZScheduler
"""

import logging
from typing import Any, Callable, Dict, List, Set

logger = logging.getLogger(__name__)

class EventEmitter:
    """
    Simple event emitter implementation for ZScheduler.

    Allows for subscribing to and emitting events throughout the system.
    Supports wildcard event listeners that receive all events.
    """

    def __init__(self):
        """Initialize a new event emitter."""
        self._listeners: Dict[str, List[Callable]] = {}
        self._wildcard_listeners: List[Callable] = []

    def on(self, event_name: str, callback: Callable) -> None:
        """
        Register an event listener.

        Args:
            event_name: Name of the event to listen for or "*" for all events
            callback: Function to call when the event occurs
        """
        if event_name == "*":
            self._wildcard_listeners.append(callback)
            return

        if event_name not in self._listeners:
            self._listeners[event_name] = []

        self._listeners[event_name].append(callback)

    def off(self, event_name: str, callback: Callable) -> bool:
        """
        Remove an event listener.

        Args:
            event_name: Name of the event to stop listening for or "*" for all events
            callback: The callback function to remove

        Returns:
            True if the listener was removed, False if not found
        """
        if event_name == "*":
            try:
                self._wildcard_listeners.remove(callback)
                return True
            except ValueError:
                return False

        if event_name not in self._listeners:
            return False

        try:
            self._listeners[event_name].remove(callback)
            return True
        except ValueError:
            return False

    def emit(self, event_name: str, *args, **kwargs) -> None:
        """
        Emit an event.

        Args:
            event_name: Name of the event to emit
            *args: Positional arguments to pass to listeners
            **kwargs: Keyword arguments to pass to listeners
        """
        # Call specific listeners
        for callback in self._listeners.get(event_name, []):
            try:
                callback(*args, **kwargs)
            except Exception as e:
                logger.exception(f"Error in event listener for {event_name}: {e}")

        # Call wildcard listeners, passing the event name as the first argument
        for callback in self._wildcard_listeners:
            try:
                callback(event_name, *args, **kwargs)
            except Exception as e:
                logger.exception(f"Error in wildcard event listener for {event_name}: {e}")

    def has_listeners(self, event_name: str) -> bool:
        """
        Check if an event has listeners.

        Args:
            event_name: Name of the event to check

        Returns:
            True if the event has listeners, False otherwise
        """
        return bool(self._listeners.get(event_name)) or bool(self._wildcard_listeners)

    def clear(self, event_name: str = None) -> None:
        """
        Clear all listeners for an event or all events.

        Args:
            event_name: Name of the event to clear or None for all events
        """
        if event_name is None:
            self._listeners = {}
            self._wildcard_listeners = []
        elif event_name == "*":
            self._wildcard_listeners = []
        else:
            self._listeners[event_name] = []



"""
Main window for ZScheduler application
"""

import sys
import logging
from typing import Optional
from datetime import datetime, timedelta

from PyQt6.QtWidgets import (
    QMainWindow, QWidget, QVBoxLayout, QHBoxLayout,
    QTabWidget, QMenu, QMenuBar, QToolBar, QStatusBar,
    QMessageBox, QFileDialog, QSystemTrayIcon, QApplication
)
from PyQt6.QtCore import Qt, QSize, pyqtSlot, pyqtSignal
from PyQt6.QtGui import QAction, QKeySequence, QCloseEvent

from zscheduler_app.scheduler.scheduler import Scheduler, Schedule
from zscheduler_app.data.json_store import JsonStore

from zscheduler_app.config.app_config import AppConfig
from zscheduler_app.themes.theme_manager import ThemeManager
from zscheduler_app.ui.schedule_list_view import ScheduleListView
from zscheduler_app.ui.calendar_view import CalendarView
from zscheduler_app.ui.timeline_view import TimelineView
from zscheduler_app.ui.statistics_dashboard import StatisticsDashboard
from zscheduler_app.ui.new_schedule_dialog import NewScheduleDialog
from zscheduler_app.ui.settings_dialog import SettingsDialog
from zscheduler_app.ui.about_dialog import AboutDialog
from zscheduler_app.ui.system_tray import SystemTrayManager
from zscheduler_app.ui.icons import IconProvider, IconManager
from zscheduler_app.themes.stylesheet import StylesheetManager

logger = logging.getLogger(__name__)

class MainWindow(QMainWindow):
    """
    Main window for ZScheduler application.

    This is the primary UI component, managing the layout and actions
    of the main application window.
    """
    # Define signals for thread-safe communication
    schedule_run_signal = pyqtSignal(str)
    schedule_complete_signal = pyqtSignal(str, bool)

    def __init__(self, app_config, scheduler, json_store):
        """
        Initialize the main window.

        Args:
            app_config: The application configuration
            scheduler: The scheduler instance
            json_store: The schedule store
        """
        super().__init__()

        self.app_config = app_config
        self.scheduler = scheduler
        self.json_store = json_store
        self._is_app_exiting = False

        # Connect signals to slots
        self.schedule_run_signal.connect(self._on_schedule_run)
        self.schedule_complete_signal.connect(self._on_schedule_complete)

        # Initialize icon provider
        self.icon_provider = IconProvider.get_instance()

        # Apply the stylesheet
        StylesheetManager.apply_theme()

        # Set window properties
        self.setWindowTitle("ZScheduler")

        # Set a fixed size appropriate for laptops - no dynamic sizing
        # These dimensions should work well on most displays
        fixed_width = 800
        fixed_height = 600

        # Set fixed size constraints for the window
        self.setMinimumSize(fixed_width, fixed_height)
        self.setMaximumSize(fixed_width, fixed_height)
        self.resize(fixed_width, fixed_height)

        # Center the window on screen
        screen_size = QApplication.primaryScreen().availableGeometry()
        self.move(
            (screen_size.width() - fixed_width) // 2,
            (screen_size.height() - fixed_height) // 2
        )

        # Set application icon and theme
        current_theme = app_config.get("ui.theme", "dark")
        self.icon_provider.set_window_icon(self, current_theme)

        # Initialize system tray if enabled
        if app_config.get("ui.minimize_to_tray", True):
            self.tray_manager = SystemTrayManager()
            self.tray_manager.show_signal.connect(self.show)
            self.tray_manager.hide_signal.connect(self.hide)
            self.tray_manager.exit_signal.connect(self._on_tray_exit)
        else:
            self.tray_manager = None

        # Initialize additional components
        self._setup_ui()
        self._create_statusbar()
        self._create_menu()
        self._create_toolbar()
        self._setup_shortcuts()

        # Update UI with current schedules
        self._update_schedules()

        # Connect scheduler callbacks using lambdas to emit signals
        if hasattr(self.scheduler, 'set_on_schedule_run_callback'):
            self.scheduler.set_on_schedule_run_callback(
                lambda schedule_id: self.schedule_run_signal.emit(schedule_id)
            )
        if hasattr(self.scheduler, 'set_on_schedule_complete_callback'):
            self.scheduler.set_on_schedule_complete_callback(
                lambda schedule_id, success: self.schedule_complete_signal.emit(schedule_id, success)
            )

        # Show status message
        self.statusBar().showMessage("Ready")

    def _setup_ui(self):
        """Set up the main UI components."""
        # Central widget
        central_widget = QWidget()
        self.setCentralWidget(central_widget)

        # Main layout
        main_layout = QVBoxLayout(central_widget)
        main_layout.setContentsMargins(10, 10, 10, 10)

        # Tab widget for different views
        self.tab_widget = QTabWidget()

        # List view
        self.list_view = ScheduleListView(self.scheduler)
        self.tab_widget.addTab(self.list_view, "List View")

        # Calendar view
        self.calendar_view = CalendarView(self.scheduler)
        self.tab_widget.addTab(self.calendar_view, "Calendar View")

        # Timeline view
        self.timeline_view = TimelineView(self.scheduler)
        self.tab_widget.addTab(self.timeline_view, "Timeline")

        # Statistics dashboard
        self.stats_dashboard = StatisticsDashboard(self.scheduler)
        self.tab_widget.addTab(self.stats_dashboard, "Statistics")

        # Connect tab changed signal
        self.tab_widget.currentChanged.connect(self._on_tab_changed)

        # Add tab widget to main layout
        main_layout.addWidget(self.tab_widget)

    def _create_menu(self):
        """Create the main menu."""
        # File menu
        file_menu = self.menuBar().addMenu("&File")

        # New schedule action
        new_action = QAction("&New Schedule...", self)
        new_action.setShortcut(QKeySequence.StandardKey.New)
        new_action.triggered.connect(self._on_new_schedule)
        file_menu.addAction(new_action)

        # Import/Export actions
        import_action = QAction("&Import Schedules...", self)
        import_action.triggered.connect(self._on_import_schedules)
        file_menu.addAction(import_action)

        export_action = QAction("&Export Schedules...", self)
        export_action.triggered.connect(self._on_export_schedules)
        file_menu.addAction(export_action)

        file_menu.addSeparator()

        # Settings action
        settings_action = QAction("&Settings...", self)
        settings_action.triggered.connect(self._on_settings)
        file_menu.addAction(settings_action)

        file_menu.addSeparator()

        # Exit action
        exit_action = QAction("E&xit", self)
        exit_action.setShortcut(QKeySequence.StandardKey.Quit)
        exit_action.triggered.connect(self.close)
        file_menu.addAction(exit_action)

        # Edit menu
        edit_menu = self.menuBar().addMenu("&Edit")

        # Edit schedule action
        self.edit_action = QAction("&Edit Schedule...", self)
        self.edit_action.setShortcut(QKeySequence.StandardKey.Open)
        self.edit_action.triggered.connect(self._on_edit_schedule)
        edit_menu.addAction(self.edit_action)

        # Duplicate schedule action
        self.duplicate_action = QAction("&Duplicate Schedule", self)
        self.duplicate_action.triggered.connect(self._on_duplicate_schedule)
        edit_menu.addAction(self.duplicate_action)

        # Delete schedule action
        self.delete_action = QAction("&Delete Schedule", self)
        self.delete_action.setShortcut(QKeySequence.StandardKey.Delete)
        self.delete_action.triggered.connect(self._on_delete_schedule)
        edit_menu.addAction(self.delete_action)

        # View menu
        view_menu = self.menuBar().addMenu("&View")

        # Refresh action
        refresh_action = QAction("&Refresh", self)
        refresh_action.setShortcut(QKeySequence.StandardKey.Refresh)
        refresh_action.triggered.connect(self._update_schedules)
        view_menu.addAction(refresh_action)

        view_menu.addSeparator()

        # Theme submenu
        theme_menu = view_menu.addMenu("&Theme")

        # Light theme action
        light_theme_action = QAction("&Light", self)
        light_theme_action.triggered.connect(lambda: self._on_theme_change("light"))
        theme_menu.addAction(light_theme_action)

        # Dark theme action
        dark_theme_action = QAction("&Dark", self)
        dark_theme_action.triggered.connect(lambda: self._on_theme_change("dark"))
        theme_menu.addAction(dark_theme_action)

        # System theme action
        system_theme_action = QAction("&System", self)
        system_theme_action.triggered.connect(lambda: self._on_theme_change("system"))
        theme_menu.addAction(system_theme_action)

        # Scheduler menu
        scheduler_menu = self.menuBar().addMenu("&Scheduler")

        # Start scheduler action
        self.start_action = QAction("&Start Scheduler", self)
        self.start_action.triggered.connect(self._on_start_scheduler)
        scheduler_menu.addAction(self.start_action)

        # Stop scheduler action
        self.stop_action = QAction("S&top Scheduler", self)
        self.stop_action.triggered.connect(self._on_stop_scheduler)
        scheduler_menu.addAction(self.stop_action)

        scheduler_menu.addSeparator()

        # Pause schedule action
        self.pause_action = QAction("&Pause Schedule", self)
        self.pause_action.triggered.connect(self._on_pause_schedule)
        scheduler_menu.addAction(self.pause_action)

        # Resume schedule action
        self.resume_action = QAction("&Resume Schedule", self)
        self.resume_action.triggered.connect(self._on_resume_schedule)
        scheduler_menu.addAction(self.resume_action)

        # Run now action
        self.run_now_action = QAction("Run &Now", self)
        self.run_now_action.triggered.connect(self._on_run_now)
        scheduler_menu.addAction(self.run_now_action)

        # Help menu
        help_menu = self.menuBar().addMenu("&Help")

        # About action
        about_action = QAction("&About ZScheduler", self)
        about_action.triggered.connect(self._on_about)
        help_menu.addAction(about_action)

        # Update action states
        self._update_action_states()

    def _create_toolbar(self):
        """Create the main toolbar."""
        # Main toolbar
        self.toolbar = QToolBar("Main Toolbar")
        self.toolbar.setIconSize(QSize(24, 24))  # Increased icon size
        self.toolbar.setObjectName("mainToolbar")
        self.toolbar.setMovable(False)
        self.addToolBar(self.toolbar)

        # New schedule action
        new_action = QAction(IconManager.new_icon(), "", self)
        new_action.setToolTip("New Schedule")
        new_action.triggered.connect(self._on_new_schedule)
        self.toolbar.addAction(new_action)

        # Edit schedule action
        edit_action = QAction(IconManager.edit_icon(), "", self)
        edit_action.setToolTip("Edit Schedule")
        edit_action.triggered.connect(self._on_edit_schedule)
        self.toolbar.addAction(edit_action)

        # Delete schedule action
        delete_action = QAction(IconManager.delete_icon(), "", self)
        delete_action.setToolTip("Delete Schedule")
        delete_action.triggered.connect(self._on_delete_schedule)
        self.toolbar.addAction(delete_action)

        self.toolbar.addSeparator()

        # Start scheduler action
        start_action = QAction(IconManager.start_icon(), "", self)
        start_action.setToolTip("Start Scheduler")
        start_action.triggered.connect(self._on_start_scheduler)
        self.toolbar.addAction(start_action)
        self.start_action = start_action

        # Stop scheduler action
        stop_action = QAction(IconManager.stop_icon(), "", self)
        stop_action.setToolTip("Stop Scheduler")
        stop_action.triggered.connect(self._on_stop_scheduler)
        self.toolbar.addAction(stop_action)
        self.stop_action = stop_action

        self.toolbar.addSeparator()

        # Pause schedule action
        pause_action = QAction(IconManager.pause_icon(), "", self)
        pause_action.setToolTip("Pause Schedule")
        pause_action.triggered.connect(self._on_pause_schedule)
        self.toolbar.addAction(pause_action)
        self.pause_action = pause_action

        # Resume schedule action
        resume_action = QAction(IconManager.resume_icon(), "", self)
        resume_action.setToolTip("Resume Schedule")
        resume_action.triggered.connect(self._on_resume_schedule)
        self.toolbar.addAction(resume_action)
        self.resume_action = resume_action

        # Run now action
        run_now_action = QAction(IconManager.run_now_icon(), "", self)
        run_now_action.setToolTip("Run Now")
        run_now_action.triggered.connect(self._on_run_now)
        self.toolbar.addAction(run_now_action)
        self.run_now_action = run_now_action

        # Show/hide toolbar based on config
        self.toolbar.setVisible(self.app_config.get("ui.show_toolbar", True))

    def _create_statusbar(self):
        """Create the status bar."""
        self.status_bar = QStatusBar()
        self.setStatusBar(self.status_bar)

        # Show/hide status bar based on config
        self.status_bar.setVisible(self.app_config.get("ui.show_statusbar", True))

    def _setup_shortcuts(self):
        """Set up keyboard shortcuts."""
        # Add Ctrl+Q to quit application
        quit_shortcut = QKeySequence("Ctrl+Q")
        quit_action = QAction(self)
        quit_action.setShortcut(quit_shortcut)
        quit_action.triggered.connect(self.close)
        self.addAction(quit_action)

    def _update_schedules(self):
        """Update the UI with current schedules."""
        schedules = self.scheduler.get_schedules()

        # Update each view
        self.list_view.update_schedules(schedules)
        self.calendar_view.update_schedules(schedules)
        self.timeline_view.update_schedules(schedules)
        self.stats_dashboard.update_statistics(schedules)

        # Update status bar
        self.status_bar.showMessage(str(len(schedules)) + " schedules loaded")

        # Update action states
        self._update_action_states()

    def _update_action_states(self):
        """Update the enabled state of actions based on current state."""
        # Get the currently selected schedule
        selected_schedule = self._get_selected_schedule()

        # Update edit/delete/duplicate actions
        has_selection = selected_schedule is not None
        self.edit_action.setEnabled(has_selection)
        self.delete_action.setEnabled(has_selection)
        self.duplicate_action.setEnabled(has_selection)

        # Update pause/resume actions
        if has_selection:
            # Check if paused, handling both Schedule objects and dictionaries
            if isinstance(selected_schedule, dict):
                is_paused = selected_schedule.get('paused', False)
            else:
                is_paused = getattr(selected_schedule, 'paused', False)

            self.pause_action.setEnabled(not is_paused)
            self.resume_action.setEnabled(is_paused)
            self.run_now_action.setEnabled(True)
        else:
            self.pause_action.setEnabled(False)
            self.resume_action.setEnabled(False)
            self.run_now_action.setEnabled(False)

        # Update scheduler actions
        is_running = self.scheduler.running
        self.start_action.setEnabled(not is_running)
        self.stop_action.setEnabled(is_running)

        # Update system tray if available
        if self.tray_manager:
            self.tray_manager.update_scheduler_state(is_running)

    def _get_selected_schedule(self) -> Optional[Schedule]:
        """
        Get the currently selected schedule from the active view.

        Returns:
            Optional[Schedule]: The selected schedule, or None if no selection
        """
        current_tab = self.tab_widget.currentIndex()

        if current_tab == 0:  # List view
            return self.list_view.get_selected_schedule()
        elif current_tab == 1:  # Calendar view
            return self.calendar_view.get_selected_schedule()
        elif current_tab == 2:  # Timeline view
            return self.timeline_view.get_selected_schedule()

        return None

    def _on_tab_changed(self, index):
        """
        Handle tab change event.

        Args:
            index: The new tab index
        """
        # Update action states for the new tab
        self._update_action_states()

    def _on_new_schedule(self):
        """Handle new schedule action."""
        dialog = NewScheduleDialog(self.scheduler, self)
        if dialog.exec():
            self._update_schedules()

    def _on_edit_schedule(self):
        """Handle edit schedule action."""
        schedule = self._get_selected_schedule()
        if schedule:
            # If we got a dictionary, we need to convert it to a Schedule object
            if isinstance(schedule, dict):
                from zscheduler_app.scheduler.scheduler import Schedule
                schedule_obj = Schedule.from_dict(schedule)
            else:
                schedule_obj = schedule

            dialog = NewScheduleDialog(self.scheduler, self, schedule_obj)
            if dialog.exec():
                self._update_schedules()

    def _on_duplicate_schedule(self):
        """Handle duplicate schedule action."""
        schedule = self._get_selected_schedule()
        if schedule:
            # Handle both Schedule objects and dictionaries
            if isinstance(schedule, dict):
                # For dictionaries, get values directly
                name = schedule.get('name', 'Unknown')
                schedule_type = schedule.get('schedule_type', 'interval')
                command = schedule.get('command', '')
                interval = schedule.get('interval', 60)
                cron_expression = schedule.get('cron_expression', '')

                # Create a new schedule with the same settings
                new_schedule_data = {
                    "name": f"Copy of {name}",
                    "command": command,
                    "schedule_type": schedule_type,
                    "interval": interval,
                    "cron_expression": cron_expression,
                    "enabled": True
                }

                self.scheduler.add_schedule(new_schedule_data)
                logger.info(f"Duplicated schedule: {name}")
            else:
                # For Schedule objects, use the existing code
                if hasattr(schedule, 'task') and schedule.task.__class__.__name__ == "CommandTask":
                    target = schedule.task.command
                else:
                    # For Python tasks, this is a simplified approach
                    target = f"{schedule.task.module_name}.{schedule.task.callable_name}" if hasattr(schedule, 'task') and schedule.task.module_name else ""

                # Create new schedule builder
                builder = self.scheduler.schedule(target).name(f"Copy of {schedule.name}")

                # Set the same trigger type
                if schedule.trigger_type == "date":
                    # For one-time schedules, set it for tomorrow at the same time
                    tomorrow = datetime.now() + timedelta(days=1)
                    builder.at(tomorrow)
                elif schedule.trigger_type == "interval":
                    # For recurring schedules, use the same interval
                    builder.every(**schedule.trigger_args)
                    if schedule.max_runs:
                        builder.repeat(schedule.max_runs)
                    builder.start_now()
                elif schedule.trigger_type == "cron":
                    # For cron schedules, use the same expression
                    cron_args = schedule.trigger_args
                    expr = f"{cron_args.get('minute', '*')} {cron_args.get('hour', '*')} {cron_args.get('day', '*')} {cron_args.get('month', '*')} {cron_args.get('day_of_week', '*')}"
                    builder.cron(expr)

                logger.info(f"Duplicated schedule: {schedule.id}")

            self._update_schedules()

    def _on_delete_schedule(self):
        """Handle delete schedule action."""
        schedule = self._get_selected_schedule()
        if schedule:
            # Get the schedule name, handling both Schedule objects and dictionaries
            schedule_name = schedule.name if hasattr(schedule, 'name') else schedule.get('name', 'Unknown')
            schedule_id = schedule.id if hasattr(schedule, 'id') else schedule.get('id')

            confirm = QMessageBox.question(
                self,
                "Confirm Delete",
                f"Are you sure you want to delete schedule '{schedule_name}'?",
                QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No
            )

            if confirm == QMessageBox.StandardButton.Yes:
                # Remove the schedule from the scheduler
                if self.scheduler.remove_schedule(schedule_id):
                    # Get the updated schedules list (without the deleted schedule)
                    updated_schedules = self.scheduler.get_schedules()

                    # Save changes to the JSON store immediately
                    try:
                        # Force overwrite the entire JSON file with the updated schedules
                        self.json_store.save(updated_schedules)

                        # Double-check that the file was updated correctly
                        # This will reload the file to verify the schedule is gone
                        loaded_schedules = self.json_store.load()
                        deleted = all(s.get('id') != schedule_id for s in loaded_schedules)

                        if deleted:
                            self.status_bar.showMessage(f"Deleted schedule: {schedule_name}")
                        else:
                            logger.error(f"Schedule {schedule_id} still exists in JSON store after deletion")
                            self.status_bar.showMessage(f"Warning: Schedule may not be fully deleted")

                    except Exception as e:
                        logger.exception(f"Error saving schedules after deletion: {e}")
                        self.status_bar.showMessage(f"Error saving after deletion: {str(e)}")

                    # Update UI
                    self._update_schedules()

    def _on_import_schedules(self):
        """Handle import schedules action."""
        file_path, _ = QFileDialog.getOpenFileName(
            self,
            "Import Schedules",
            "",
            "JSON Files (*.json)"
        )

        if file_path:
            try:
                # Import schedules from JSON file
                import_store = JsonStore(file_path)
                imported_schedules = import_store.load_schedules()

                # Check for schedules with same name
                current_names = {s.name for s in self.scheduler.get_schedules() if s.name}
                duplicates = [s for s in imported_schedules if s.name in current_names]

                if duplicates:
                    # Ask how to handle duplicates
                    duplicate_msg = f"Found {len(duplicates)} schedules with existing names. How would you like to proceed?"
                    duplicate_dialog = QMessageBox(
                        QMessageBox.Icon.Question,
                        "Duplicate Schedules",
                        duplicate_msg,
                        QMessageBox.StandardButton.NoButton,
                        self
                    )
                    skip_button = duplicate_dialog.addButton("Skip Duplicates", QMessageBox.ButtonRole.AcceptRole)
                    rename_button = duplicate_dialog.addButton("Rename", QMessageBox.ButtonRole.ActionRole)
                    replace_button = duplicate_dialog.addButton("Replace", QMessageBox.ButtonRole.DestructiveRole)
                    cancel_button = duplicate_dialog.addButton("Cancel Import", QMessageBox.ButtonRole.RejectRole)

                    duplicate_dialog.exec()
                    clicked = duplicate_dialog.clickedButton()

                    if clicked == cancel_button:
                        return
                    elif clicked == skip_button:
                        imported_schedules = [s for s in imported_schedules if s.name not in current_names]
                    elif clicked == rename_button:
                        for schedule in duplicates:
                            schedule.name = f"Imported {schedule.name}"
                    # If replace, we'll let them overwrite

                # Add the imported schedules
                for schedule in imported_schedules:
                    self.scheduler.add_schedule(schedule)

                self._update_schedules()
                self.status_bar.showMessage(f"Imported {len(imported_schedules)} schedules from {file_path}")

            except Exception as e:
                logger.exception(f"Error importing schedules: {e}")
                QMessageBox.critical(
                    self,
                    "Import Error",
                    f"Failed to import schedules: {str(e)}"
                )

    def _on_export_schedules(self):
        """Handle export schedules action."""
        file_path, _ = QFileDialog.getSaveFileName(
            self,
            "Export Schedules",
            "",
            "JSON Files (*.json)"
        )

        if file_path:
            try:
                # Create an export JSON store and save schedules
                export_store = JsonStore(file_path)
                schedules = self.scheduler.get_schedules()
                export_store.save_schedules(schedules)

                self.status_bar.showMessage(f"Exported {len(schedules)} schedules to {file_path}")

            except Exception as e:
                logger.exception(f"Error exporting schedules: {e}")
                QMessageBox.critical(
                    self,
                    "Export Error",
                    f"Failed to export schedules: {str(e)}"
                )

    def _on_settings(self):
        """Handle settings action."""
        dialog = SettingsDialog(self.app_config, self)
        if dialog.exec():
            # Update UI based on new settings
            self.toolbar.setVisible(self.app_config.get("ui.show_toolbar", True))
            self.status_bar.setVisible(self.app_config.get("ui.show_statusbar", True))

            # Apply the stylesheet with any theme changes
            StylesheetManager.apply_theme()

    def _on_theme_change(self, theme_name):
        """
        Handle theme change.

        Args:
            theme_name: The name of the theme to apply
        """
        # Apply the stylesheet
        StylesheetManager.apply_theme()

        # Save the theme preference
        self.app_config.set("ui.theme", theme_name)
        self.app_config.save()

        # Update system tray icon if available
        if self.tray_manager:
            self.tray_manager.update_icon(theme_name == "dark")

        # Update application icon
        self.icon_provider.set_window_icon(self, theme_name)

    def _on_start_scheduler(self):
        """Handle start scheduler action."""
        self.scheduler.start()
        self._update_action_states()
        self.status_bar.showMessage("Scheduler started")

        if self.tray_manager and self.app_config.get("notifications.enabled", True):
            self.tray_manager.show_notification(
                "ZScheduler",
                "Scheduler started",
                QSystemTrayIcon.MessageIcon.Information
            )

    def _on_stop_scheduler(self):
        """Handle stop scheduler action."""
        self.scheduler.stop()
        self._update_action_states()
        self.status_bar.showMessage("Scheduler stopped")

        if self.tray_manager and self.app_config.get("notifications.enabled", True):
            self.tray_manager.show_notification(
                "ZScheduler",
                "Scheduler stopped",
                QSystemTrayIcon.MessageIcon.Information
            )

    def _on_pause_schedule(self):
        """Handle pause schedule action."""
        schedule = self._get_selected_schedule()
        if schedule:
            # Get the schedule name and id, handling both Schedule objects and dictionaries
            schedule_name = schedule.name if hasattr(schedule, 'name') else schedule.get('name', 'Unknown')
            schedule_id = schedule.id if hasattr(schedule, 'id') else schedule.get('id')

            self.scheduler.pause_schedule(schedule_id)
            self._update_schedules()
            self.status_bar.showMessage(f"Paused schedule: {schedule_name}")

    def _on_resume_schedule(self):
        """Handle resume schedule action."""
        schedule = self._get_selected_schedule()
        if schedule:
            # Get the schedule name and id, handling both Schedule objects and dictionaries
            schedule_name = schedule.name if hasattr(schedule, 'name') else schedule.get('name', 'Unknown')
            schedule_id = schedule.id if hasattr(schedule, 'id') else schedule.get('id')

            self.scheduler.resume_schedule(schedule_id)
            self._update_schedules()
            self.status_bar.showMessage(f"Resumed schedule: {schedule_name}")

    def _on_run_now(self):
        """Handle run now action."""
        schedule = self._get_selected_schedule()
        if schedule:
            # Get the schedule name and id, handling both Schedule objects and dictionaries
            schedule_name = schedule.name if hasattr(schedule, 'name') else schedule.get('name', 'Unknown')

            # In a real implementation, this would execute the schedule immediately
            if hasattr(schedule, 'execute'):
                schedule.execute()
            else:
                # If it's a dictionary, we need to find the actual schedule object
                schedule_id = schedule.get('id')
                if schedule_id:
                    self.scheduler._execute_schedule(schedule_id)

            self.status_bar.showMessage(f"Running schedule: {schedule_name}")
            self._update_schedules()

    def _on_about(self):
        """Handle about action."""
        dialog = AboutDialog(self)
        dialog.exec()

    def _toggle_window_visibility(self):
        """Toggle the visibility of the main window."""
        if self.isVisible():
            self.hide()
        else:
            self.show()
            self.activateWindow()

        # Update tray icon text if available
        if self.tray_manager:
            self.tray_manager.update_show_hide_text(self.isVisible())

    def _toggle_scheduler(self):
        """Toggle the scheduler state."""
        if self.scheduler.running:
            self._on_stop_scheduler()
        else:
            self._on_start_scheduler()

    @pyqtSlot(str)
    def _on_schedule_run(self, schedule_id):
        """
        Handle schedule run event. This is a slot connected to a signal
        for thread-safe operation.

        Args:
            schedule_id: The ID of the schedule being run
        """
        self._update_schedules()

    @pyqtSlot(str, bool)
    def _on_schedule_complete(self, schedule_id, success):
        """
        Handle schedule complete event. This is a slot connected to a signal
        for thread-safe operation.

        Args:
            schedule_id: The ID of the completed schedule
            success: Whether the schedule executed successfully
        """
        self._update_schedules()

    def closeEvent(self, event: QCloseEvent):
        """
        Handle window close event.

        Args:
            event: The close event
        """
        if self.tray_manager and self.app_config.get("general.minimize_to_tray", True) and not self._is_app_exiting:
            # Minimize to tray instead of closing
            event.ignore()
            self.hide()

            # Show notification if first time
            if not hasattr(self, '_minimized_to_tray_before'):
                self._minimized_to_tray_before = True
                self.tray_manager.show_notification(
                    "ZScheduler",
                    "ZScheduler is still running in the background. Click the tray icon to restore.",
                    QSystemTrayIcon.MessageIcon.Information
                )
        else:
            # No need to save window size since we use fixed dimensions

            # Stop the scheduler first
            try:
                logger.info("Stopping scheduler from main window close event...")
                self.scheduler.stop()
            except Exception as e:
                logger.exception(f"Error stopping scheduler on exit: {e}")

            # Save schedules
            try:
                self.json_store.save(self.scheduler.get_schedules())
            except Exception as e:
                logger.exception(f"Error saving schedules on exit: {e}")

            # Accept the close event
            event.accept()

    def set_app_exiting(self):
        """Mark the application as exiting to prevent minimize to tray."""
        self._is_app_exiting = True

    def _on_tray_exit(self):
        """Handle exit signal from system tray."""
        self.set_app_exiting()
        self.close()



"""
Calendar view for ZScheduler application
"""

import logging
from typing import List, Optional
from datetime import datetime, timedelta

from PyQt6.QtWidgets import (
    QWidget, QVBoxLayout, QCalendarWidget, QListWidget,
    QListWidgetItem, QLabel, QSplitter
)
from PyQt6.QtCore import Qt, QDate, pyqtSignal

from zscheduler_app.scheduler.scheduler import Scheduler

logger = logging.getLogger(__name__)

class CalendarView(QWidget):
    """
    Calendar view component for ZScheduler.

    Displays schedules on a calendar interface, allowing users to
    see scheduled tasks by date and select them for viewing or editing.
    """

    # Signal emitted when a schedule is selected
    schedule_selected = pyqtSignal(str)

    def __init__(self, scheduler=None, parent=None):
        """
        Initialize the calendar view.

        Args:
            scheduler: Scheduler instance
            parent: Parent widget
        """
        super().__init__(parent)
        self.scheduler = scheduler
        self.schedules = []
        self.selected_schedule = None

        self._setup_ui()

    def _setup_ui(self):
        """Set up the UI components."""
        # Main layout
        layout = QVBoxLayout(self)

        # Splitter for calendar and task list
        splitter = QSplitter(Qt.Orientation.Vertical)

        # Calendar widget
        self.calendar = QCalendarWidget()
        self.calendar.setGridVisible(True)
        self.calendar.setVerticalHeaderFormat(QCalendarWidget.VerticalHeaderFormat.NoVerticalHeader)

        # Use SingleSelection instead of SingleDate which may not be available in some PyQt6 versions
        try:
            self.calendar.setSelectionMode(QCalendarWidget.SelectionMode.SingleDate)
        except AttributeError:
            try:
                self.calendar.setSelectionMode(QCalendarWidget.SelectionMode.SingleSelection)
            except AttributeError:
                # Fallback to integer value if enum is not available
                self.calendar.setSelectionMode(1)  # 1 corresponds to SingleSelection/SingleDate

        self.calendar.clicked.connect(self.on_date_selected)

        # Current date is selected by default
        self.selected_date = QDate.currentDate()
        self.calendar.setSelectedDate(self.selected_date)

        # Task list for selected day
        self.task_list_container = QWidget()
        task_list_layout = QVBoxLayout(self.task_list_container)

        self.day_label = QLabel(self.selected_date.toString("dddd, MMMM d, yyyy"))
        task_list_layout.addWidget(self.day_label)

        self.task_list = QListWidget()
        self.task_list.setAlternatingRowColors(True)
        self.task_list.itemClicked.connect(self.on_task_selected)
        task_list_layout.addWidget(self.task_list)

        # Add widgets to splitter
        splitter.addWidget(self.calendar)
        splitter.addWidget(self.task_list_container)

        # Set splitter sizes
        splitter.setSizes([2, 1])  # 2:1 ratio

        # Add splitter to main layout
        layout.addWidget(splitter)

    def update_schedules(self, schedules: List[dict]) -> None:
        """
        Update the view with new schedules.

        Args:
            schedules: List of schedule dictionaries to display
        """
        self.schedules = schedules

        # Update the calendar
        self._highlight_dates_with_schedules()

        # Update the task list for the currently selected date
        self._update_task_list()

    def on_date_selected(self, date: QDate):
        """
        Handle date selection event.

        Args:
            date: Selected date
        """
        self.selected_date = date
        self.day_label.setText(date.toString("dddd, MMMM d, yyyy"))
        self._update_task_list()

    def on_task_selected(self, item: QListWidgetItem):
        """
        Handle task selection event.

        Args:
            item: Selected task item
        """
        schedule_id = item.data(Qt.ItemDataRole.UserRole)
        for schedule in self.schedules:
            if schedule.get('id') == schedule_id:
                self.selected_schedule = schedule
                self.schedule_selected.emit(schedule_id)
                break

    def _highlight_dates_with_schedules(self):
        """Highlight dates that have scheduled tasks."""
        # In a real implementation, we would highlight dates on the calendar
        # that have scheduled tasks. This is a placeholder.
        pass

    def _update_task_list(self):
        """Update the task list for the selected date."""
        self.task_list.clear()

        if not self.schedules:
            return

        # Get Python date from QDate
        py_date = datetime(
            self.selected_date.year(),
            self.selected_date.month(),
            self.selected_date.day()
        )

        # Find schedules for the selected date
        for schedule in self.schedules:
            next_run = schedule.get('next_run')
            if not next_run:
                continue

            try:
                next_run_dt = datetime.fromisoformat(next_run)
                if py_date.date() == next_run_dt.date():
                    name = schedule.get('name', '')
                    if not name:
                        schedule_id = schedule.get('id', '')
                        if schedule_id:
                            name = "Task-" + schedule_id[:8]
                        else:
                            name = "Task"
                    time_str = next_run_dt.strftime('%H:%M:%S')
                    item = QListWidgetItem(name + " - " + time_str)
                    item.setData(Qt.ItemDataRole.UserRole, schedule.get('id'))
                    self.task_list.addItem(item)
            except (ValueError, TypeError) as e:
                logger.error("Error parsing next_run time: " + str(e))

    def get_selected_schedule(self) -> Optional[dict]:
        """
        Get the currently selected schedule.

        Returns:
            The selected schedule dictionary or None if no schedule is selected
        """
        return self.selected_schedule



"""
PythonTask class for executing Python functions and objects
"""

import inspect
import logging
import pickle
import base64
from typing import Any, Callable, Dict, List, Optional, Tuple, Union

from .task import Task

logger = logging.getLogger(__name__)

class PythonTask(Task):
    """
    Task for executing Python functions and objects.

    This task can execute:
    - Plain Python functions and methods
    - Object methods (storing a reference to the object)
    - Module-level functions
    - Class methods
    """

    def __init__(
        self,
        target: Union[Callable, Any],
        *args,
        **kwargs
    ):
        """
        Initialize a new Python task.

        Args:
            target: The function, method, or object to call
            *args: Positional arguments to pass to the target
            **kwargs: Keyword arguments to pass to the target
        """
        # Determine name from target
        if hasattr(target, "__name__"):
            name = f"Python: {target.__name__}"
        elif hasattr(target, "__class__") and hasattr(target.__class__, "__name__"):
            name = f"Python: {target.__class__.__name__}"
        else:
            name = f"Python: {str(target)[:30]}"

        super().__init__(name)
        self.target = target
        self.args = args
        self.kwargs = kwargs

        # Determine if target is a method on an object
        self.is_bound_method = hasattr(target, "__self__") and inspect.ismethod(target)

        # Determine if target is callable
        self.is_callable = callable(target)

        # Store info about the function/method for serialization
        if callable(target):
            self.callable_name = getattr(target, "__name__", None)
            self.module_name = getattr(target.__module__, "__name__", None) if hasattr(target, "__module__") else None
            self.qualname = getattr(target, "__qualname__", None)
        else:
            self.callable_name = None
            self.module_name = None
            self.qualname = None

    def execute(self) -> Any:
        """
        Execute the Python function or method.

        Returns:
            The return value from the called function
        """
        logger.info(f"Executing Python task: {self.name}")

        try:
            if self.is_callable:
                result = self.target(*self.args, **self.kwargs)
                return result
            else:
                raise ValueError(f"Target is not callable: {self.target}")

        except Exception as e:
            logger.exception(f"Error executing Python task: {e}")
            raise

    def to_dict(self) -> Dict[str, Any]:
        """
        Convert the task to a dictionary for serialization.

        Note: This can only fully serialize certain types of targets.
        Complex objects or closures may not serialize properly.

        Returns:
            A dictionary representation of the task
        """
        data = super().to_dict()

        # Try to pickle the target, args, and kwargs
        try:
            pickled_target = base64.b64encode(pickle.dumps(self.target)).decode("utf-8")
            pickled_args = base64.b64encode(pickle.dumps(self.args)).decode("utf-8")
            pickled_kwargs = base64.b64encode(pickle.dumps(self.kwargs)).decode("utf-8")

            data.update({
                "pickled_target": pickled_target,
                "pickled_args": pickled_args,
                "pickled_kwargs": pickled_kwargs,
                "is_pickled": True
            })
        except Exception as e:
            # Pickling failed, store info about the function instead
            logger.warning(f"Could not pickle Python task: {e}")

            data.update({
                "callable_name": self.callable_name,
                "module_name": self.module_name,
                "qualname": self.qualname,
                "is_bound_method": self.is_bound_method,
                "is_pickled": False,
                # Provide a string representation of args and kwargs for reference
                "args_repr": repr(self.args),
                "kwargs_repr": repr(self.kwargs)
            })

        return data

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> 'PythonTask':
        """
        Create a task from a dictionary representation.

        Args:
            data: Dictionary containing task data

        Returns:
            A new PythonTask instance or raises ValueError if cannot be deserialized
        """
        # Check if the data contains a pickled target
        if data.get("is_pickled", False):
            try:
                target = pickle.loads(base64.b64decode(data["pickled_target"]))
                args = pickle.loads(base64.b64decode(data["pickled_args"]))
                kwargs = pickle.loads(base64.b64decode(data["pickled_kwargs"]))

                task = cls(target, *args, **kwargs)

                if "id" in data:
                    task.id = data["id"]
                if "name" in data:
                    task.name = data["name"]

                return task

            except Exception as e:
                raise ValueError(f"Failed to unpickle Python task: {e}")
        else:
            # Without pickle data, we need to try to find the function by name
            module_name = data.get("module_name")
            callable_name = data.get("callable_name")

            if module_name and callable_name:
                try:
                    import importlib
                    module = importlib.import_module(module_name)
                    target = getattr(module, callable_name)

                    # Create task with empty args/kwargs
                    task = cls(target)

                    if "id" in data:
                        task.id = data["id"]
                    if "name" in data:
                        task.name = data["name"]

                    logger.warning(f"Created Python task for {module_name}.{callable_name} but without original arguments")
                    return task

                except (ImportError, AttributeError) as e:
                    raise ValueError(f"Failed to import Python task target: {e}")
            else:
                raise ValueError("Cannot deserialize Python task: missing pickled data and module/callable information")




"""
Schedule builder for ZScheduler - provides fluent API for schedule configuration
"""

from datetime import datetime, timedelta
from typing import Any, Callable, Dict, List, Optional, Union, TYPE_CHECKING

import dateutil.parser
from croniter import croniter

from ..tasks.task import Task
from ..tasks.command_task import CommandTask
from ..tasks.python_task import PythonTask
from .schedule import Schedule

if TYPE_CHECKING:
    from .scheduler import Scheduler

class ScheduleBuilder:
    """
    Fluent API for building schedules.

    This class provides a fluent interface for configuring schedules,
    allowing for readable and concise schedule definitions.
    """

    def __init__(self, scheduler: 'Scheduler', target: Union[str, Callable, Any], *args, **kwargs):
        """
        Initialize a new schedule builder.

        Args:
            scheduler: The scheduler instance
            target: The target to schedule (command string, function, etc.)
            *args: Positional arguments to pass to the target function
            **kwargs: Keyword arguments to pass to the target function
        """
        self._scheduler = scheduler
        self._target = target
        self._args = args
        self._kwargs = kwargs
        self._task = self._create_task(target, *args, **kwargs)
        self._name: Optional[str] = None
        self._trigger_type: str = "date"
        self._trigger_args: Dict[str, Any] = {}
        self._max_runs: Optional[int] = None

    def _create_task(self, target: Union[str, Callable, Any], *args, **kwargs) -> Task:
        """
        Create a task for the given target.

        Args:
            target: The target to schedule
            *args: Positional arguments
            **kwargs: Keyword arguments

        Returns:
            A Task instance suitable for the target
        """
        if isinstance(target, str):
            return CommandTask(target)
        else:
            return PythonTask(target, *args, **kwargs)

    def _build_schedule(self) -> str:
        """
        Build and add the schedule to the scheduler.

        Returns:
            The schedule ID
        """
        schedule = Schedule(
            task=self._task,
            name=self._name,
            trigger_type=self._trigger_type,
            trigger_args=self._trigger_args,
            max_runs=self._max_runs
        )
        return self._scheduler._add_schedule(schedule)

    def name(self, name: str) -> 'ScheduleBuilder':
        """
        Set a name for this schedule.

        Args:
            name: The schedule name

        Returns:
            Self for method chaining
        """
        self._name = name
        return self

    def at(self, dt: Union[str, datetime]) -> str:
        """
        Schedule to run once at a specific date and time.

        Args:
            dt: The date and time to run (string or datetime)

        Returns:
            The schedule ID
        """
        if isinstance(dt, str):
            dt = dateutil.parser.parse(dt)

        self._trigger_type = "date"
        self._trigger_args = {"run_date": dt}
        return self._build_schedule()

    def after(self, **kwargs) -> str:
        """
        Schedule to run once after a specified time interval.

        Args:
            **kwargs: Time interval (seconds, minutes, hours, days, weeks)

        Returns:
            The schedule ID

        Examples:
            >>> scheduler.schedule(func).after(minutes=5)
            >>> scheduler.schedule(func).after(hours=2, minutes=30)
        """
        delta = timedelta(**kwargs)
        run_date = datetime.now() + delta

        self._trigger_type = "date"
        self._trigger_args = {"run_date": run_date}
        return self._build_schedule()

    def every(self, **kwargs) -> 'ScheduleBuilder':
        """
        Schedule to run at a regular interval.

        Args:
            **kwargs: Interval (seconds, minutes, hours, days, weeks)

        Returns:
            Self for method chaining

        Examples:
            >>> scheduler.schedule(func).every(minutes=5).start_now()
            >>> scheduler.schedule(func).every(hours=1).repeat(5)
        """
        self._trigger_type = "interval"
        self._trigger_args = kwargs
        return self

    def start_now(self) -> str:
        """
        Start the recurring schedule immediately.

        Returns:
            The schedule ID
        """
        return self._build_schedule()

    def start_at(self, dt: Union[str, datetime]) -> str:
        """
        Start the recurring schedule at a specific time.

        Args:
            dt: Start date/time (string or datetime)

        Returns:
            The schedule ID
        """
        if isinstance(dt, str):
            dt = dateutil.parser.parse(dt)

        self._trigger_args["start_date"] = dt
        return self._build_schedule()

    def repeat(self, count: int) -> 'ScheduleBuilder':
        """
        Set a maximum number of times to run.

        Args:
            count: Maximum number of runs

        Returns:
            Self for method chaining
        """
        self._max_runs = count
        return self

    def cron(self, expr: str) -> str:
        """
        Schedule using a cron expression.

        Args:
            expr: Cron expression (e.g., "0 12 * * *" for daily at noon)

        Returns:
            The schedule ID
        """
        if not croniter.is_valid(expr):
            raise ValueError(f"Invalid cron expression: {expr}")

        self._trigger_type = "cron"

        # Parse the cron expression into APScheduler arguments
        parts = expr.split()
        if len(parts) == 5:
            minute, hour, day, month, day_of_week = parts
        elif len(parts) == 6:
            minute, hour, day, month, day_of_week, year = parts
        else:
            raise ValueError("Invalid cron expression format")

        self._trigger_args = {
            "minute": minute,
            "hour": hour,
            "day": day,
            "month": month,
            "day_of_week": day_of_week
        }

        if len(parts) == 6:
            self._trigger_args["year"] = year

        return self._build_schedule()

    def daily_at(self, time_str: str) -> str:
        """
        Schedule to run daily at a specific time.

        Args:
            time_str: Time in "HH:MM" format

        Returns:
            The schedule ID
        """
        hour, minute = time_str.split(":")

        self._trigger_type = "cron"
        self._trigger_args = {
            "minute": minute,
            "hour": hour,
            "day": "*",
            "month": "*",
            "day_of_week": "*"
        }

        return self._build_schedule()

    def weekly_on(self, day: Union[int, str], time: str) -> str:
        """
        Schedule to run weekly on a specific day and time.

        Args:
            day: Day of the week (0-6 or name like "mon", "tue")
            time: Time in "HH:MM" format

        Returns:
            The schedule ID
        """
        hour, minute = time.split(":")

        self._trigger_type = "cron"
        self._trigger_args = {
            "minute": minute,
            "hour": hour,
            "day": "*",
            "month": "*",
            "day_of_week": str(day)
        }

        return self._build_schedule()

    def monthly_on(self, day: int, time: str) -> str:
        """
        Schedule to run monthly on a specific day and time.

        Args:
            day: Day of the month (1-31)
            time: Time in "HH:MM" format

        Returns:
            The schedule ID
        """
        hour, minute = time.split(":")

        self._trigger_type = "cron"
        self._trigger_args = {
            "minute": minute,
            "hour": hour,
            "day": str(day),
            "month": "*",
            "day_of_week": "*"
        }

        return self._build_schedule()



import argparse
import sys
from pathlib import Path
from zscheduler_app.config.app_config import AppConfig
from zscheduler_app.data.json_store import JsonStore
from zscheduler_app.scheduler.scheduler import Scheduler
from datetime import datetime
import shlex
import json

# Import browser_launcher if available
try:
    from browser_launcher.profile_manager import BrowserProfileManager, LaunchOptions
except ImportError:
    BrowserProfileManager = None
    LaunchOptions = None

CONFIG_PATH = str(Path.home() / ".zscheduler" / "config.json")
SCHEDULES_PATH = str(Path.home() / ".zscheduler" / "schedules.json")

def main():
    parser = argparse.ArgumentParser(description="ZScheduler CLI")
    subparsers = parser.add_subparsers(dest="command")

    # Add browser task
    add_browser = subparsers.add_parser("add-browser-task", help="Add a browser launch schedule")
    add_browser.add_argument("--browser", required=True, help="Browser type (e.g., firefox, chrome)")
    add_browser.add_argument("--profile", required=True, help="Browser profile name or path")
    add_browser.add_argument("--url", required=True, help="URL to open")
    add_browser.add_argument("--name", default=None, help="Task name")
    add_browser.add_argument("--interval", type=int, default=None, help="Interval in seconds (for recurring)")
    add_browser.add_argument("--once", default=None, help="Run once at datetime (ISO format)")
    add_browser.add_argument("--cron", default=None, help="Cron expression (for cron schedule)")
    add_browser.add_argument("--incognito", action="store_true", help="Launch in incognito/private mode")
    add_browser.add_argument("--enabled", action="store_true", help="Enable the schedule (default: enabled)")

    # List schedules
    list_parser = subparsers.add_parser("list", help="List all schedules")

    # Run schedule now
    run_parser = subparsers.add_parser("run", help="Run a schedule immediately")
    run_parser.add_argument("schedule_id", help="ID of the schedule to run")

    # Remove schedule
    remove_parser = subparsers.add_parser("remove", help="Remove a schedule by ID")
    remove_parser.add_argument("schedule_id", help="ID of the schedule to remove")

    args = parser.parse_args()

    # Load config and schedules
    app_config = AppConfig(CONFIG_PATH)
    json_store = JsonStore(SCHEDULES_PATH)
    scheduler = Scheduler()
    schedules = json_store.load()
    for sched in schedules:
        scheduler.add_schedule(sched)

    if args.command == "add-browser-task":
        # Store the command as a JSON-encoded list of arguments
        cmd_args = [
            "launch_browser",
            "--browser", args.browser,
            "--profile", args.profile,
            "--url", args.url
        ]
        if args.incognito:
            cmd_args.append("--incognito")
        cmd = json.dumps(cmd_args)
        name = args.name or f"{args.browser} {args.profile} {args.url}"
        enabled = args.enabled or True
        schedule_type = "interval" if args.interval else "once" if args.once else "cron" if args.cron else "interval"
        schedule_data = {
            "name": name,
            "command": cmd,
            "schedule_type": schedule_type,
            "enabled": enabled,
        }
        if args.interval:
            schedule_data["interval"] = args.interval
        if args.once:
            try:
                dt = datetime.fromisoformat(args.once)
                schedule_data["cron_expression"] = dt.isoformat()
            except Exception:
                print("Invalid datetime format for --once. Use ISO format.")
                sys.exit(1)
        if args.cron:
            schedule_data["cron_expression"] = args.cron
        schedule_id = scheduler.add_schedule(schedule_data)
        json_store.save(scheduler.get_schedules())
        print(f"Added browser schedule with ID: {schedule_id}")

    elif args.command == "list":
        all_scheds = scheduler.get_schedules()
        if not all_scheds:
            print("No schedules found.")
        else:
            for sched in all_scheds:
                print(f"ID: {sched['id']} | Name: {sched['name']} | Type: {sched['schedule_type']} | Command: {sched['command']} | Enabled: {sched['enabled']}")

    elif args.command == "run":
        sched = scheduler.get_schedule(args.schedule_id)
        if not sched:
            print(f"Schedule with ID {args.schedule_id} not found.")
            sys.exit(1)
        cmd = sched["command"]
        try:
            cmd_args = json.loads(cmd)
        except Exception:
            print(f"Invalid command format for schedule {args.schedule_id}. Cannot run.")
            sys.exit(1)
        if cmd_args and cmd_args[0] == "launch_browser":
            if BrowserProfileManager is None:
                print("browser_launcher library is not installed. Cannot launch browser.")
                sys.exit(1)
            parser = argparse.ArgumentParser()
            parser.add_argument("--browser", required=True)
            parser.add_argument("--profile", required=True)
            parser.add_argument("--url", required=True)
            parser.add_argument("--incognito", action="store_true")
            try:
                args2 = parser.parse_args(cmd_args[1:])
            except Exception as e:
                print(f"Failed to parse browser launch command: {e}")
                sys.exit(1)
            mgr = BrowserProfileManager()
            options = LaunchOptions(incognito=args2.incognito)
            print(f"Launching {args2.browser} with profile '{args2.profile}' and URL '{args2.url}'" + (" in incognito mode" if args2.incognito else ""))
            mgr.launch_browser(
                browser_type=args2.browser,
                profile=args2.profile,
                url=args2.url,
                options=options
            )
        else:
            print(f"Would execute: {cmd}")
            # Here you could add support for other command types
    elif args.command == "remove":
        removed = scheduler.remove_schedule(args.schedule_id)
        if removed:
            json_store.save(scheduler.get_schedules())
            print(f"Removed schedule with ID: {args.schedule_id}")
        else:
            print(f"Schedule with ID {args.schedule_id} not found.")
    else:
        parser.print_help()

if __name__ == "__main__":
    main()
